﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using IronBreaker.Core.Utilities;
using Styx;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Inventory;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using M = IronBreaker.Main;

namespace IronBreaker.Core.Helpers
{
    internal static class Common
    {
        private static LocalPlayer Me => StyxWoW.Me;
        private static readonly HashSet<ArtifactPower> ArtifactPowers = new HashSet<ArtifactPower>();

        #region [Tasks] - Rotation Selections

        public static async Task<bool> SpecSelectorRotation()
        {
            if (Me.Specialization == WoWSpec.WarriorArms)
                return await Rotations.Arms.RotationSelector();
            if (Me.Specialization == WoWSpec.WarriorFury)
                return await Rotations.Fury.RotationSelector();
            if (Me.Specialization == WoWSpec.WarriorProtection)
                return await Rotations.Protection.RotationSelector();

            await Movement.ClearTheDead();
            return false;
        }

        public static async Task<bool> SpecSelectorPreCombatBuffBehavior()
        {
            if (Me.Specialization == WoWSpec.WarriorArms)
                return await Rotations.Arms.PreCombatBuffing();
            if (Me.Specialization == WoWSpec.WarriorFury)
                return await Rotations.Fury.PreCombatBuffing();
            if (Me.Specialization == WoWSpec.WarriorProtection)
                return await Rotations.Protection.PreCombatBuffing();
            await Movement.ClearTheDead();
            return false;
        }

        public static async Task<bool> SpecSelectorCombatBuffBehavior()
        {
            if (Me.Specialization == WoWSpec.WarriorArms)
                await Rotations.Arms.CombatBuffing();
            if (Me.Specialization == WoWSpec.WarriorFury)
                await Rotations.Fury.CombatBuffing();
            if (Me.Specialization == WoWSpec.WarriorProtection)
                await Rotations.Protection.CombatBuffing();

            await Movement.ClearTheDead();
            return false;
        }

        public static async Task<bool> SpecSelectorPullBehavior()
        {
            if (Me.Specialization == WoWSpec.WarriorArms)
                await Rotations.Arms.Pull();
            if (Me.Specialization == WoWSpec.WarriorFury)
                await Rotations.Fury.Pull();
            if (Me.Specialization == WoWSpec.WarriorProtection)
                await Rotations.Protection.Pull();
            await Movement.ClearTheDead();
            return false;
        }

        public static async Task<bool> ForceCombat()
        {
            if (Me.Specialization == WoWSpec.WarriorArms)
                return await Rotations.Arms.RotationSelector();
            if (Me.Specialization == WoWSpec.WarriorFury)
                return await Rotations.Fury.RotationSelector();
            if (Me.Specialization == WoWSpec.WarriorProtection)
                return await Rotations.Protection.RotationSelector();

            return false;
        }

        public static async Task<bool> RestSelector()
        {
            
            if (M.IS.IgnorePvP && M.IS.ForceCombat && !Unit.PvPUnit(Me.CurrentTarget) && Unit.IsValidUnit(Me.CurrentTarget) && Me.GotTarget && Me.CurrentTarget.IsValidCombatUnit() && !Me.Combat && !Me.OnTaxi && !Me.Mounted && !Me.IsDead)
                await ForceCombat();
            if (!M.IS.IgnorePvP && M.IS.ForceCombat && Me.GotTarget && Unit.IsValidUnit(Me.CurrentTarget) && Me.CurrentTarget.IsValidCombatUnit() && !Me.OnTaxi && !Me.Combat && !Me.Mounted && !Me.IsDead)
                await ForceCombat();

            if (Me.HasAura("Resurrection Sickness")) { await Rotations.Sickness.HandleRezSickness(); }
            if (NeedToRest() && CanRest() && Consumable.GetBestFood(false) != null) { await RestBehavior(); }
            await CommonCoroutines.SleepForLagDuration();
            return false;
        }

        public static async Task<bool> RestBehavior()
        {
            return await Rotations.Rest.RestBehavior();
        }

        #endregion

        #region [Method] - Checks

        public static bool NeedToRest()
        {
            return M.IS.EnableResting && M.IS.RestingValue > 0 && Me.HealthPercent < M.IS.RestingValue;
        }

        public static bool CanRest()
        {
            return !Me.Mounted && !Me.IsDead && !Me.IsGhost && !Me.IsOnTransport && !Me.OnTaxi && !Me.Combat;
        }

        #endregion

        #region [Method] - Cache

        #region Declarations
        public static uint CurrentRage;
        public static uint MaxRage;
        public static uint DrainedRage;
        #endregion

        public static void Cache()
        {
            CurrentRage = GetRage();
            MaxRage = GetMaxRage();
            DrainedRage = MaxRage > CurrentRage ? MaxRage - CurrentRage : 0;
        }

        #endregion

        #region [Method] - Get Power

        public static uint GetRage()
        {
            try
            {
                using (StyxWoW.Memory.AcquireFrame())
                    return Lua.GetReturnVal<uint>("return UnitPower(\"player\");", 0);
            }
            catch (Exception ex)
            {
                Log.DebugWrite("Exception in GetRage() Method", ex);
                return Me.CurrentPower;
            }
        }

        public static uint GetMaxRage()
        {
            try
            {
                using (StyxWoW.Memory.AcquireFrame())
                    return Lua.GetReturnVal<uint>("return UnitPowerMax(\"player\");", 0);
            }
            catch (Exception ex)
            {
                Log.DebugWrite("Exception thrown in GetMaxRage() Method", ex);
                return Me.CurrentPower;
            }
        }

        #endregion

        #region [Method] - Artifact Power

        private static bool HasArtifactWeaponEquip => Lua.GetReturnVal<bool>("return HasArtifactEquipped()", 0);
        private static bool IsValid => Lua.GetReturnVal<bool>("return ArtifactFrame~=nil", 0);
        private static bool ArtifactFrameVisable => Lua.GetReturnVal<bool>("return ArtifactFrame:IsShown()", 0);
        private static void OpenArtifactFrame() { Lua.DoString(@"SocketInventoryItem(16)"); }
        private static void CloseArtifactFrame() { Lua.DoString("HideUIPanel(ArtifactFrame)"); }

        public static void UpdateArtifactPowers()
        {
            ArtifactPowers.Clear();
            if (!HasArtifactWeaponEquip)
            {
                Log.DebugWrite("No Artifact Equip");
                return;
            }

            while (!IsValid || !ArtifactFrameVisable)
            {
                OpenArtifactFrame();
            }

            using (StyxWoW.Memory.AcquireFrame())
            {
                var ids = Lua.GetReturnValues("return unpack(C_ArtifactUI.GetPowers())");

                foreach (var i in ids)
                {
                    var id = Lua.ParseLuaValue<int>(i);
                    var info = Lua.GetReturnValues($"return C_ArtifactUI.GetPowerInfo({id})");
                    var spellId = Lua.ParseLuaValue<int>(info[0]);
                    var name = Lua.GetReturnVal<string>($"return GetSpellInfo({spellId})", 0);

                    var ap = new ArtifactPower(
                        spellId,
                        name,
                        Lua.ParseLuaValue<int>(info[1]),
                        Lua.ParseLuaValue<int>(info[2]),
                        Lua.ParseLuaValue<int>(info[3]),
                        Lua.ParseLuaValue<int>(info[4]));

                    Log.DebugWrite($"Artifact Power {ap}");
                    ArtifactPowers.Add(ap);
                }
                CloseArtifactFrame();
            }
        }

        public static int CurrentArtifactRank(int id)
        {
            return ArtifactPowers.FirstOrDefault(a => a.SpellId == id)?.CurrentRank ?? 0;
        }

        #endregion
    }
}
