﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace IronBreaker.Core.Helpers
{
    internal static class Extensions
    {
        private static LocalPlayer Me => StyxWoW.Me;

        // Method to check if our target is a valid target to attack
        public static bool IsValidCombatUnit(this WoWUnit unit)
        {
            return unit != null && unit.IsValid && unit.IsAlive && unit.Attackable && !unit.IsCritter;
        }

        public static float MeleeDistance(this WoWUnit unit, WoWUnit target = null)
        {
            if (unit == null)
                return 0;
            if (target == null) throw new ArgumentNullException(nameof(target));
            if (target.IsMe)
            {
                target = unit;
                unit = StyxWoW.Me;
            }
            if (target.IsPlayer && unit.IsPlayer)
                return 3.5f;
            return Math.Max(5f, target.CombatReach + 1.3333334f);
        }

        public static float MeleeDistance(this WoWUnit unit)
        {
            return Me.MeleeDistance(unit);
        }

        // Method to check if "Me" has a specified aura active
        public static bool HasAura(this LocalPlayer me, IEnumerable<int> auraList)
        {
            return auraList.Any(me.HasAura);
        }

        public static bool HasAnyAura(this WoWUnit unit, params string[] auraNames)
        {
            var auras = unit.GetAllAuras();
            var hashes = new HashSet<string>(auraNames);
            return auras.Any(a => hashes.Contains(a.Name));
        }

        public static bool HasAnyTempAura(this WoWUnit unit, params string[] auraNames)
        {
            IEnumerable<WoWAura> auras = unit.GetAllAuras().Where(a => !a.IsPassive);
            HashSet<string> hashes = new HashSet<string>(auraNames);
            return auras.Any(a => hashes.Contains(a.Name));
        }

        public static bool IsTrivial(this WoWUnit unit)
        {
            if (unit == null)
                return false;
            if (unit.Elite)
                return unit.Level <= 10;
            return unit.Level <= 10;
        }

        // Method to add space(s) to the inputted string "data" 
        public static string AddSpaces(this string data)
        {
            var r = new Regex(@"(?!^)(?=[A-Z])");
            return r.Replace(data, " ");
        }

        // Method to return a null safe int using "uint.TryParse" on the data string
        public static uint SafeGetInt(this string data)
        {
            try
            {
                uint safeInt;
                return uint.TryParse(data, out safeInt) ? safeInt : 0;
            }
            catch
            {
                return 0;
            }
        }

        // Method to "attempt" at removing a key value from a "Concurrent Dictionary" collection.
        public static bool TryRemove<TKey, TValue>(this ConcurrentDictionary<TKey, TValue> self, TKey key)
        {
            TValue ignore;
            return self.TryRemove(key, out ignore);
        }

        public static bool IsTrainingDummyBoss(this WoWUnit unit)
        {
            List<string> trainingDummyNames = new List<string> { "Imprisoned Centurion", "Imprisoned Weaver" };
            return trainingDummyNames.Contains(unit.SafeName);
        }

        public static bool IsTrainingDummyElite(this WoWUnit unit)
        {
            List<string> trainingDummyNames = new List<string> { "Imprisoned Centurion", "Imprisoned Weaver" };
            return trainingDummyNames.Contains(unit.SafeName);
        }

        public static string CamelFormat(this string str)
        {
            var newString = new StringBuilder();
            foreach (char c in str)
            {
                if (char.IsUpper(c))
                    newString.Append(' ');
                newString.Append(c);
            }
            return newString.ToString();
        }
    }
}
