﻿using System.Collections.Generic;
using System.Linq;
using Styx;
using Styx.CommonBot;
using Styx.WoWInternals.WoWObjects;

namespace IronBreaker.Core.Helpers
{
    internal static class Group
    {
        const int ProvingGrounds = 6852;
        const int Protector = 72218;
        const int MistWeaver = 71828;

        public static List<WoWUnit> Tanks
        {
            get
            {
                if (!StyxWoW.Me.GroupInfo.IsInParty)
                    return new List<WoWUnit>();
                if (StyxWoW.Me.ZoneId == ProvingGrounds)
                    return Unit.GroupMembers.Where(u => u.Entry == Protector).ToList();

                return StyxWoW.Me.GroupInfo.RaidMembers.Where(p => p.HasRole(WoWPartyMember.GroupRole.Tank))
                    .Select(p => p.ToPlayer())
                    .Union(new[] { RaFHelper.Leader })
                    .Where(p => p != null && p.IsValid)
                    .Distinct()
                    .ToList<WoWUnit>();
            }
        }

        public static List<WoWUnit> Healers
        {
            get
            {
                if (!StyxWoW.Me.GroupInfo.IsInParty)
                    return new List<WoWUnit>();

                if (StyxWoW.Me.ZoneId == ProvingGrounds)
                {
                    WoWUnit healer = Unit.GroupMembers.FirstOrDefault(u => u.Entry == MistWeaver);
                    if (healer == null && Unit.GroupMembers.Any(u => !u.IsMe))
                        healer = StyxWoW.Me;
                    if (healer == null)
                        return new List<WoWUnit>();
                    return new List<WoWUnit>(new[] { healer });
                }

                return StyxWoW.Me.GroupInfo.RaidMembers.Where(p => p.HasRole(WoWPartyMember.GroupRole.Healer))
                    .Select(p => p.ToPlayer())
                    .Where(p => p != null)
                    .ToList<WoWUnit>();
            }
        }

        public static List<WoWUnit> Dps
        {
            get
            {
                if (!StyxWoW.Me.GroupInfo.IsInParty)
                    return new List<WoWUnit>();

                return
                    StyxWoW.Me.GroupInfo.RaidMembers.Where(
                        p => !p.HasRole(WoWPartyMember.GroupRole.Tank) && !p.HasRole(WoWPartyMember.GroupRole.Healer))
                        .Select(p => p.ToPlayer())
                        .Where(p => p != null)
                        .ToList<WoWUnit>();
            }
        }

        public static bool MeTank
        {
            get
            {
                if (StyxWoW.Me.ZoneId == ProvingGrounds)
                    return Unit.GroupMembers.Any(u => u.Entry == MistWeaver);

                return (StyxWoW.Me.Role & WoWPartyMember.GroupRole.Tank) != 0
                    || Tanks.All(t => !t.IsAlive);
            }
        }

        public static bool TankNearby
        {
            get { return Tanks.Any(h => h.IsAlive && h.Distance < 40); }
        }

        public static bool HealerNearby
        {
            get { return Healers.Any(h => h.IsAlive && h.Distance < 40); }
        }

        public static WoWUnit GetPlayerByClass(float range, bool isDead, params WoWClass[] classes)
        {
            return (from woWClass in classes
                    select StyxWoW.Me.GroupInfo.RaidMembers.FirstOrDefault(
                        p => p.ToPlayer() != null && p.ToPlayer().Distance < range && p.ToPlayer().Class == woWClass)
                        into unit
                    where unit != null
                    where !isDead && unit.Dead || unit.Ghost
                    select unit.ToPlayer()).FirstOrDefault();
        }
    }
}
