﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using IronBreaker.Core.Utilities;
using IronBreaker.Settings;
using Styx.Helpers;

namespace IronBreaker.Core.Helpers
{
    class Update
    {
        private static readonly string IronBreakerSvnUrl = "https://acroroutines.svn.cloudforge.com/ironbreaker/";
        private static readonly Regex Link = new Regex(@"<li><a href="".+"">(?<ln>.+(?:..))</a></li>", RegexOptions.CultureInvariant);
        private static string RemoveXmlEscapes(string xml) => xml.Replace("&amp;", "&").Replace("&lt;", "<").Replace("&gt;", ">").Replace("&quot;", "\"").Replace("&apos;", "'");

        public static void CheckForUpdate() => UpdateCheck(Styx.Common.Utilities.AssemblyDirectory + "\\Routines\\IronBreaker\\");

        static void UpdateCheck(string path)
        {
            try
            {
                Log.Write(Coloring.Info, "Checking if we have the latest update.");
                var latestRev = GetRevision();

                if (IronBreakerSettings.Instance.CurrentRevision != latestRev)
                {
                    Log.Write(Coloring.Info, "[Updater] A new update is available!\r\n");
                    DownloadSvnFiles(new WebClient(), IronBreakerSvnUrl, path);
                    IronBreakerSettings.Instance.CurrentRevision = latestRev;
                    IronBreakerSettings.Instance.Save();
                    Log.Write(Coloring.Info, "****************************************");
                    Log.Write(Coloring.Info, "IronBreaker Updated. Restart Honorbuddy");
                    Log.Write(Coloring.Info, "****************************************");
                }
                else
                {
                    Log.Write(Coloring.Info, "[Updater] We are using the latest version!");
                }
            }
            catch (Exception ex)
            {
                Log.Write("Error retrieving Update: {0}", ex);
            }
        }

        private static string GetRevision()
        {
            try
            {
                var wc = new WebClient();
                string webData = wc.DownloadString(IronBreakerSvnUrl + "version");
                Log.Write(Coloring.Info, "SVN Version: {0}", webData);
                return webData;
            }
            catch (Exception ex)
            {
                Log.Write(Coloring.Cancel, "Exception thrown in GetRevision(): {0}", ex);
                return "0";
            }
        }

        private static void DownloadSvnFiles(WebClient client, string url, string path)
        {
            var html = client.DownloadString(url);
            MatchCollection results = Link.Matches(html);
            IEnumerable<Match> matches = from match in results.OfType<Match>()
                where match.Success && match.Groups["ln"].Success
                select match;

            foreach (Match match in matches)
            {
                var file = RemoveXmlEscapes(match.Groups["ln"].Value);
                var newUrl = url + file;
                if (newUrl[newUrl.Length - 1] == '/')
                    DownloadSvnFiles(client, newUrl, path);
                else
                {
                    string filePath, dirPath;
                    if (url.Length > IronBreakerSvnUrl.Length)
                    {
                        var relativePath = url.Substring(IronBreakerSvnUrl.Length);
                        dirPath = Path.Combine(path, relativePath);
                        filePath = Path.Combine(dirPath, file);
                    }
                    else
                    {
                        dirPath = Environment.CurrentDirectory;
                        filePath = Path.Combine(path, file);
                    }

                    Log.Write(Coloring.Info, "[Updater] Downloading {0}.", file);

                    try
                    {
                        if (!Directory.Exists(dirPath))
                            Directory.CreateDirectory(dirPath);
                        client.DownloadFile(newUrl, filePath);
                        Log.Write(Coloring.Info, "[Updater] Download {0} Done.", file);
                    }
                    catch (Exception ex)
                    {
                        Log.Write(Coloring.Info, "DownloadSvnFiles Error: {0}.", ex);
                    }
                }
            }
        }
    }
}
