﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Frames;
using Styx.WoWInternals.WoWObjects;

namespace IronBreaker.Core
{
    internal class Item
    {
        private static LocalPlayer Me => StyxWoW.Me;

        public static async Task<bool> UseItem(WoWItem item, bool reqs, string log = null)
        {
            if (item == null) { return false; }
            if (!reqs) { return false; }
            if (!CanUseItem(item)) { return false; }

            //Log.UseItemLog(string.Format($"/use {item.Name}" + (string.IsNullOrEmpty(log) || !M.IS.EnableDebugLog ? "" : " - " + log)), Coloring.Info);
            item.Use();
            await CommonCoroutines.SleepForLagDuration();
            return true;
        }

        public static WoWItem GetItemByName(string name)
        {
            return Me.CarriedItems.Where(b => b.SafeName == name)
                     .OrderBy(l => l.ItemInfo.Level)
                     .ThenByDescending(b => b.ItemInfo.RequiredSkillLevel)
                     .FirstOrDefault();
        }

        public static WoWItem FindFirstUsableItemBySpell(params string[] names)
        {
            List<WoWItem> itemsCarried = StyxWoW.Me.CarriedItems;
            var nameHash = new HashSet<string>(names);

            return (from i in itemsCarried
                    let items = i.Effects
                    where IsUsableItem(i, nameHash)
                    orderby i.ItemInfo.Level descending
                    select i)
                    .FirstOrDefault();
        }

        public static bool IsUsableItem(WoWItem i, HashSet<string> nameHash)
        {
            return i.Usable
                && i.Cooldown == 0
                && i.ItemInfo != null
                && i.ItemInfo.RequiredLevel <= StyxWoW.Me.Level
                && (i.ItemInfo.RequiredSkillId == 0 || i.ItemInfo.RequiredSkillLevel <= Me.GetSkill(i.ItemInfo.RequiredSkillId).CurrentValue)
                && (i.ItemInfo.RequiredSpellId == 0 || SpellManager.HasSpell(i.ItemInfo.RequiredSpellId) || Me.HasAura(i.ItemInfo.RequiredSpellId));
        }

        public static WoWItem FindFood()
        {
            return Me.CarriedItems
                     .Where(f => f.ItemInfo.ItemClass == WoWItemClass.Consumable
                     && f.ItemInfo.ConsumableClass == WoWItemConsumableClass.FoodAndDrink
                     && (f.ItemInfo.RequiredSkillId == 0 || Me.GetSkill(f.ItemInfo.RequiredSkillId).CurrentValue >= f.ItemInfo.RequiredSkillLevel)
                     && f.ItemInfo.RequiredLevel <= Me.Level
                     && CanUseItem(f))
                     .OrderBy(l => l.ItemInfo.Level)
                     .ThenByDescending(f => f.ItemInfo.RequiredSkillLevel)
                     .FirstOrDefault();
        }

        #region [Method] - Item Checks

        // Method to check if Item is usable + Cooldown is greater than 0 + no merchant window is open on wow
        private static bool CanUseItem(WoWItem item)
        {
            return item.Usable && item.Cooldown <= 0 && !MerchantFrame.Instance.IsVisible;
        }

        #endregion
    }
}
