﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using Styx;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace IronBreaker.Core.Managers
{
    internal class TankManager : Targeting
    {
        public new static TankManager Instance { get; set; }
        public List<WoWUnit> NeedToTaunt { get; private set; }
        public static bool NeedTankTargeting { get; set; }
        public static readonly WaitTimer TargetingTimer = new WaitTimer(TimeSpan.FromSeconds(1));
        protected override List<WoWObject> GetInitialObjectList() { return ObjectManager.GetObjectsOfType<WoWUnit>().Cast<WoWObject>().ToList(); }
        private static List<WoWUnit> EnemyTauntCount { get; }

        static TankManager()
        {
            Instance = new TankManager { NeedToTaunt = new List<WoWUnit>() };
        }

        public override void Pulse()
        {
            if (NeedTankTargeting)
                base.Pulse();
        }

        private static IEnumerable<WoWUnit> SurroundingEnemies()
        {
            return ObjectManager.GetObjectsOfTypeFast<WoWUnit>();
        }

        public static IEnumerable<WoWUnit> ActiveEnemiesToTaunt(Vector3 location, double range)
        {
            var hostile = EnemyTauntCount;
            return hostile?.Where(x => x.Location.DistanceSquared(location) <= range * range);
        }

        public static void EnemiesToTauntRange(double range)
        {
            EnemyTauntCount.Clear();

            foreach (var target in SurroundingEnemies())
            {
                if (target == null || !target.IsValid)
                    continue;
                if (!target.IsAlive || target.DistanceSqr > range * range)
                    continue;
                if (!target.Attackable || target.CanSelect)
                    continue;
                if (target.IsFriendly)
                    continue;
                if (target.IsNonCombatPet && target.IsCritter)
                    continue;

                if (target.IsTargetingMyPartyMember || target.IsTargetingMyRaidMember)
                    EnemyTauntCount.Add(target);
            }
        }

        protected override void DefaultRemoveTargetsFilter(List<WoWObject> units)
        {
            for (int i = units.Count - 1; i >= 0; i--)
            {
                if (!units[i].IsValid)
                {
                    units.RemoveAt(i);
                    continue;
                }

                WoWUnit u = units[i].ToUnit();

                if (u.IsFriendly || u.IsDead || u.IsPet || !u.Combat || u.IsCrowedControlled())
                {
                    units.RemoveAt(i);
                    continue;
                }

                if (u.DistanceSqr > 40 * 40)
                {
                    units.RemoveAt(i);
                    continue;
                }

                if (Unit.MobIgnoreList.Contains(u.Entry))
                {
                    units.RemoveAt(i);
                    continue;
                }

                if (u.CurrentTarget == null)
                    continue;

                WoWUnit tar = u.CurrentTarget;
                if (!tar.IsPlayer || !tar.IsHostile)
                    continue;

                units.RemoveAt(i);
            }
        }

        private static int GetAggroDifferenceFor(WoWUnit unit, IEnumerable<WoWPlayer> partyMembers)
        {
            uint myThreat = unit.ThreatInfo.ThreatValue;
            uint highestParty = (from p in partyMembers
                                 let tVal = unit.GetThreatInfoFor(p).ThreatValue
                                 orderby tVal descending
                                 select tVal)
                .FirstOrDefault();

            int result = (int)myThreat - (int)highestParty;
            return result;
        }

        protected override void DefaultTargetWeight(List<TargetPriority> units)
        {
            NeedToTaunt.Clear();
            List<WoWPlayer> members = StyxWoW.Me.GroupInfo.IsInRaid ? StyxWoW.Me.RaidMembers : StyxWoW.Me.PartyMembers;

            foreach (TargetPriority priority in units)
            {
                WoWUnit u = priority.Object.ToUnit();
                int aggroDiff = GetAggroDifferenceFor(u, members);
                priority.Score -= aggroDiff;

                if (aggroDiff < 0 && !u.Fleeing && !u.IsPlayer)
                {
                    NeedToTaunt.Add(u);
                }
            }
        }
    }
}
