﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using IronBreaker.Core.Helpers;
using IronBreaker.Core.Utilities;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace IronBreaker.Core
{
    internal static class Unit
    {
        #region [Method] - Declarations
        private static LocalPlayer Me => StyxWoW.Me;
        private static List<WoWUnit> EnemyCount { get; }
        private static List<WoWUnit> EnemyTauntCount { get; }
        public static HashSet<uint> MobIgnoreList = new HashSet<uint> { 52288, 52302, 52320, 52525, 52387 };
        private const int AllianceBanner = 61573;
        private const int HordeBanner = 61574;
        #endregion

        private static IEnumerable<WoWUnit> SurroundingEnemies() { return ObjectManager.GetObjectsOfTypeFast<WoWUnit>(); }
        public static IEnumerable<WoWUnit> ActiveEnemies(Vector3 location, double range)
        {
            var hostile = EnemyCount;
            return hostile?.Where(x => x.Location.DistanceSquared(location) <= range * range);
        }
        public static IEnumerable<WoWUnit> ActiveEnemiesToTaunt(Vector3 location, double range)
        {
            var hostile = EnemyTauntCount;
            return hostile?.Where(x => x.Location.DistanceSquared(location) <= range * range);
        }

        static Unit()
        {
            EnemyCount = new List<WoWUnit>();
            EnemyTauntCount = new List<WoWUnit>();
        }

        #region [Method] - Active Enemies

        public static void EnemyAnnex(double range)
        {
            EnemyCount.Clear();
            foreach (var unit in SurroundingEnemies())
            {
                if (unit == null || !unit.IsValid)
                    continue;

                if (unit.IsFlyingOrCannotReach())
                    continue;

                if (!unit.IsAlive || unit.DistanceSqr > range * range)
                    continue;

                if (!unit.Attackable || !unit.CanSelect)
                    continue;

                if (unit.IsFriendly)
                    continue;

                if (Main.IS.IgnorePvP && PvPUnit(unit))
                    continue;
                if (unit.IsNonCombatPet || unit.IsCritter)
                    continue;

                if (unit.IsTargetingMeOrPet || unit.IsTargetingMyPartyMember || unit.IsTargetingMyRaidMember || unit.IsTrainingDummy() || unit.IsTrainingDummyBoss() || unit.IsTrainingDummyElite())
                    EnemyCount.Add(unit);
            }
        }

        #endregion

        #region [Method] - Taunt Units

        public static void TauntAnnex(double range)
        {
            EnemyTauntCount.Clear();
            foreach (var unit in SurroundingEnemies())
            {
                if (unit == null || !unit.IsValid)
                    continue;
                //if (PvPUnit(unit))
                //    continue;
                if (!unit.IsAlive || unit.DistanceSqr > range * range)
                    continue;
                if (!unit.Attackable || !unit.CanSelect)
                    continue;
                if (unit.IsFriendly)
                    continue;
                if (unit.IsNonCombatPet || unit.IsCritter)
                    continue;

                if ((unit.IsTargetingMyPartyMember || unit.IsTargetingMyRaidMember) && !unit.IsTargetingMeOrPet)
                    EnemyTauntCount.Add(unit);
            }
        }

        #endregion

        #region [Method] - Aura Helpers

        public static bool AuraExists(WoWUnit unit, int auraId, bool isMyAura = false)
        {
            try
            {
                if (unit == null || !unit.IsValid)
                    return false;

                var aura = isMyAura
                    ? unit.GetAllAuras().FirstOrDefault(a => a.SpellId == auraId && a.CreatorGuid == Me.Guid)
                    : unit.GetAllAuras().FirstOrDefault(a => a.SpellId == auraId);
                return aura != null;
            }
            catch (Exception ex)
            {
                Log.DiagnosticLog("Exception in AuraExists(); Method.", ex);
                return false;
            }
        }

        public static uint AuraStackCount(WoWUnit unit, int auraId, bool isMyAura = false)
        {
            try
            {
                if (unit == null || !unit.IsValid)
                    return 0;

                var aura = isMyAura
                    ? unit.GetAllAuras().FirstOrDefault(a => a.SpellId == auraId && a.CreatorGuid == Me.Guid)
                    : unit.GetAllAuras().FirstOrDefault(a => a.SpellId == auraId);
                return aura?.StackCount ?? 0;
            }
            catch (Exception ex)
            {
                Log.DiagnosticLog("Exception in AuraStackCount(); Method.", ex);
                return 0;
            }
        }

        public static double AuraTimeLeft(WoWUnit unit, int auraId, bool isMyAura = false)
        {
            try
            {
                if (unit == null || !unit.IsValid)
                    return 0;

                var aura = isMyAura
                    ? unit.GetAllAuras().FirstOrDefault(a => a.SpellId == auraId && a.CreatorGuid == Me.Guid)
                    : unit.GetAllAuras().FirstOrDefault(a => a.SpellId == auraId);

                if (aura != null)
                    return aura.TimeLeft.TotalMilliseconds;

                return 0;
            }
            catch (Exception ex)
            {
                Log.DiagnosticLog("Exception in AuraTimeLeft(); Method.", ex);
                return 9999;
            }
        }

        public static bool HasAuraWithEffect(this WoWUnit unit, WoWApplyAuraType applyType)
        {
            return unit.Auras.Values.Any(a => a.Spell != null && a.Spell.SpellEffects.Any(se => applyType == se.AuraType));
        }

        public static bool HasAuraWithEffect(this WoWUnit unit, params WoWApplyAuraType[] applyType)
        {
            var hashes = new HashSet<WoWApplyAuraType>(applyType);
            return unit.Auras.Values.Any(a => a.Spell != null && a.Spell.SpellEffects.Any(se => hashes.Contains(se.AuraType)));
        }

        #endregion

        #region [Method] - Unit Getters

        public static WoWPlayer GetUnitParent(WoWUnit unit)
        {
            WoWUnit unitParent = unit;
            while (true)
            {
                if (unitParent.OwnedByUnit != null)
                    unitParent = unitParent.OwnedByRoot;
                else if (unitParent.CreatedByUnit != null) { unitParent = unitParent.CreatedByUnit; }
                else if (unitParent.SummonedByUnit != null) { unitParent = unitParent.SummonedByUnit; }
                else { break; }
            }
            if (unit != unitParent && unitParent.IsPlayer) { return unitParent as WoWPlayer; }
            return null;
        }

        #endregion
        
        #region [Method] - Unit Checks

        public static bool PvPUnit(WoWUnit unit)
        {
            if (unit != null)
            {
                if (unit.IsAlive && unit.IsHostile && unit.IsPlayer && unit.Attackable)
                {
                    WoWPlayer player = unit.ToPlayer();
                    if ((player != null) && ((Me.IsHorde && !player.IsHorde) || (Me.IsAlliance && !player.IsAlliance)))
                        return true;
                }
            }
            return false;
        }

        public static bool IsUnit(WoWObject o)
        {
            if (o != null)
            {
                try
                {
                    if (o.ToUnit() != null)
                        return true;
                }
                catch (Exception)
                {
                }
            }
            return false;
        }

        public static bool IsPlayerBossOrElite(this WoWUnit unit)
        {
            return unit.IsPlayer || unit.IsBoss || unit.Elite;
        }

        public static bool IsUnitEvading(this WoWUnit unit)
        {
            return (unit.Flags & 0x10) != 0;
        }

        public static bool IsFlyingOrCannotReach(this WoWUnit unit)
        {
            if (!StyxWoW.Me.GotTarget())
                return false;

            if (StyxWoW.Me.CurrentTarget.IsWithinMeleeRange)
                return false;

            if (StyxWoW.Me.CurrentTarget.IsPlayer)
                return false;

            float targetHeight = StyxWoW.Me.CurrentTarget.HeightOffGround();
            float meleeDistance = StyxWoW.Me.CurrentTarget.MeleeDistance();
            if (targetHeight > meleeDistance)
            {
                if (Main.IS.EnableDebugLog)
                    Log.DebugWrite(Coloring.Debug, "Unit: {0} is {1:F3} yds in the air. Attempting the use of ranged attacks to pull it down.", StyxWoW.Me.CurrentTarget.SafeName, targetHeight);
                return true;
            }

            double heightCheck = StyxWoW.Me.CurrentTarget.MeleeDistance();
            if (StyxWoW.Me.CurrentTarget.Distance2DSqr < heightCheck*heightCheck &&
                Math.Abs(StyxWoW.Me.Z - StyxWoW.Me.CurrentTarget.Z) >= heightCheck)
            {
                if (Main.IS.EnableDebugLog)
                    Log.Write(Coloring.Debug, "Unit: {0} is off the ground! Attempting the use of Ranged Attacks", StyxWoW.Me.CurrentTarget.SafeName);
                return true;
            }

            if (!StyxWoW.Me.CurrentTarget.IsWithinMeleeRange && !Movement.CanNavigateToMelee(StyxWoW.Me.CurrentTarget))
            {
                if (Main.IS.EnableDebugLog)
                    Log.Write(Coloring.Debug, "Unable to navigate to melee range on unit: {0}", StyxWoW.Me.CurrentTarget.SafeName);
                return true;
            }
            return false;
        }

        public static float HeightOffGround(this WoWUnit u)
        {
            var unitLoc = new Vector3(u.Location.X, u.Location.Y, u.Location.Z);
            float zBelow = u.FindGroundBelow();
            if (zBelow == float.MaxValue) { return float.MaxValue; }
            return unitLoc.Z - zBelow;
        }

        public static float FindGroundBelow(this WoWUnit unit)
        {
            var unitLoc = unit.Location;
            return unitLoc.Z;
        }

        public static bool GotTarget(this WoWUnit unit)
        {
            return unit.CurrentTarget != null;
        }

        public static bool IsIgnorableCritter(this WoWUnit unit)
        {
            if (!unit.IsCritter)
                return false;
            if (Styx.CommonBot.POI.BotPoi.Current.Guid == unit.Guid && Styx.CommonBot.POI.BotPoi.Current.Type == Styx.CommonBot.POI.PoiType.Kill)
                return false;
            if (Targeting.Instance.TargetList.Contains(unit))
                return false;
            if (unit.ThreatInfo.ThreatValue != 0 && unit.IsTargetingMyRaidMember)
                return false;
            return true;
        }

        public static bool IsTrainingDummy(this WoWUnit unit)
        {
            int bannerId = StyxWoW.Me.IsHorde ? AllianceBanner : HordeBanner;
            return unit != null && unit.Level > 1 && ((unit.CurrentHealth == 1 && unit.MaxHealth < unit.Level) || unit.HasAura(bannerId) || unit.Name.Contains("Training Dummy"));
        }

        public static bool IsCrowedControlled(this WoWUnit unit)
        {
            Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;
            return unit.Stunned || unit.Rooted || unit.Fleeing || unit.HasAuraWithEffect(
                WoWApplyAuraType.ModConfuse,
                WoWApplyAuraType.ModCharm,
                WoWApplyAuraType.ModFear,
                WoWApplyAuraType.ModPacify,
                WoWApplyAuraType.ModPacifySilence,
                WoWApplyAuraType.ModPossess,
                WoWApplyAuraType.ModRoot,
                WoWApplyAuraType.ModStun);
        }

        public static bool AbleToAttack(this WoWUnit unit, bool initialFocus = true)
        {
            if (unit == null) { return false; }
            bool attackable;
            if (unit.Guid == StyxWoW.Me.CurrentTargetGuid)
                attackable = Lua.GetReturnVal<bool>("return UnitCanAttack(\"player\",\"target\")", 0);
            else
            {
                if (Main.CurrentContext != WoWContext.Normal) { return true; }
                if (!unit.IsPlayer) { return true; }
                WoWUnit saveFocus = StyxWoW.Me.FocusedUnit;
                StyxWoW.Me.SetFocus(unit);
                attackable = Lua.GetReturnVal<bool>("return UnitCanAttack(\"player\",\"focus\")", 0);

                if (initialFocus)
                {
                    if (saveFocus == null || !saveFocus.IsValid) { StyxWoW.Me.SetFocus(WoWGuid.Empty); }
                    else { StyxWoW.Me.SetFocus(saveFocus); }
                }
            }
            return attackable;
        }

        public static bool IsValidUnit(WoWUnit unit, bool log = false)
        {
            if (unit == null || !unit.IsValid)
                return false;

            if (StyxWoW.Me.IsInInstance && MobIgnoreList.Contains(unit.Entry))
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Contained in our IgnoreMobsList", unit.SafeName); }
                return false;
            }

            if (!unit.CanSelect)
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unable to select.", unit.SafeName); }
                return false;
            }

            if (!unit.Attackable)
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unable to attack.", unit.SafeName); }
                return false;
            }

            if (unit.IsDead)
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. It is dead.", unit.SafeName); }
                return false;
            }

            if (unit.IsPlayer)
            {
                WoWPlayer player = unit.ToPlayer();
                if (player.IsHorde == StyxWoW.Me.IsHorde && !unit.IsHostile)
                {
                    if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Non-Hostile Player", unit.SafeName); }
                    return false;
                }

                if (!player.AbleToAttack())
                {
                    if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unable to attack.", unit.SafeName); }
                    return false;
                }
                return true;
            }

            if (unit.IsUnitEvading())
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unit is evading.", unit.SafeName); }
                return false;
            }

            if (unit.IsFriendly)
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unit is friendly.", unit.SafeName); }
                return false;
            }

            if (unit.IsTrainingDummy() || unit.IsBoss) { return true; }
            WoWPlayer playerParent = GetUnitParent(unit);

            if (playerParent != null && playerParent.IsAlive && !playerParent.IsMe)
            {
                if (!IsValidUnit(playerParent))
                {
                    if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. The pets owner is not attackable", unit.SafeName); }
                    return false;
                }

                if (!StyxWoW.Me.IsPvPFlagged)
                {
                    if (log) { Log.Write(Coloring.Cancel, "{0} is valid, but we are not flagged.", unit.SafeName); }
                    return false;
                }
                return true;
            }

            if (unit.IsNonCombatPet)
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unit is a Non-Combat Pet", unit.SafeName); }
                return false;
            }

            if (IsIgnorableCritter(unit))
            {
                if (log) { Log.Write(Coloring.Cancel, "{0} is invalid. Unit is a critter.", unit.SafeName); }
                return false;
            }

            return true;
        }

        #endregion

        #region [Method] - Group Units

        public static bool IsInGroup(this LocalPlayer me)
        {
            return me.GroupInfo.IsInParty || me.GroupInfo.IsInRaid;
        }

        public static IEnumerable<WoWUnit> GroupMembers
        {
            get
            {
                HashSet<WoWGuid> guids = new HashSet<WoWGuid>(StyxWoW.Me.GroupInfo.RaidMemberGuids) { StyxWoW.Me.Guid };
                List<WoWUnit> list =
                    ObjectManager.ObjectList.Where(o => IsUnit(o) && guids.Contains(o.Guid))
                        .Select(o => o.ToUnit())
                        .ToList();
                return list;
            }
        }

        public static IEnumerable<WoWUnit> GroupMembersPets
        {
            get
            {
                HashSet<WoWGuid> guids = new HashSet<WoWGuid>(StyxWoW.Me.GroupInfo.RaidMemberGuids);
                List<WoWUnit> list =
                    ObjectManager.ObjectList.Where(
                        o =>
                            IsUnit(o) &&
                            (guids.Contains(o.Guid) ||
                             (o.ToUnit().IsPet && guids.Contains(o.ToUnit().SummonedByUnitGuid))))
                        .Select(o => o.ToUnit())
                        .ToList();
                return list;
            }
        }

        #endregion
    }
}
