﻿using System;
using System.Collections.Concurrent;
using IronBreaker.Settings;
using Styx;
using Styx.Common;
using Styx.Helpers;
using Styx.WoWInternals.WoWObjects;

#region ClassRedundancy
using M = IronBreaker.Main;
using Color = System.Drawing.Color;
#endregion

namespace IronBreaker.Core.Utilities
{
    public static class Coloring
    {
        public static Color Normal = Color.Crimson;
        public static Color HighNormal = Color.White;
        public static Color SpellNonHeal = Color.DodgerBlue;
        public static Color Debug = Color.Orange;
        public static Color Diagnostic = Color.Yellow;
        public static Color Cancel = Color.Red;
        public static Color Init = Color.Cyan;
        public static Color Targeting = Color.LightCoral;
        public static Color Info = Color.LightPink;
        public static Color Combat = Color.Chartreuse;
        public static Color Tank = Color.MediumSpringGreen;
        public static Color Interrupt = Color.DeepPink;
    }

    internal class Log
    {
        #region Declarations
        private static LocalPlayer Me => StyxWoW.Me;
        private static WoWUnit CurrentTarget => StyxWoW.Me.CurrentTarget;
        protected static ConcurrentDictionary<string, DateTime> MMessages = new ConcurrentDictionary<string, DateTime>();
        //private static string CurrentPowerDescription => Me.Specialization == WoWSpec.DemonHunterHavoc ? "Fury" : "Pain";
        public static string LastInfoMsg;
        public static string LastWriteMessage;
        public static string LastItemUseMsg;
        public static string LastDefenseLogMsg;
        public static string LastCombatMsg;
        public static string LastDebugMsg;
        static int logLineNo;
        #endregion

        #region [Method] - Writing

        /// <summary>
        /// Writes a message in "Gold" to HB Output window and our routines log file
        /// </summary>
        /// <param name="message">message to write</param>
        public static void Write(string message)
        {
            if (LastWriteMessage == message)
                return;
            Write(Color.Gold, message);
            LastWriteMessage = message;
        }

        /// <summary>
        /// Writes a message in "Gold" to HB Output window and our routines log file
        /// </summary>
        /// <param name="message">Message to write, embedded parameters</param>
        /// <param name="args">Replacement parameter value</param>
        public static void Write(string message, params object[] args)
        {
            if (LastWriteMessage == message)
                return;
            Write(Color.Gold, message, args);
            LastWriteMessage = message;
        }

        /// <summary>
        /// Writes a message to HB Output window nad our routines log file
        /// Limits lines outputted by checking that new lines differ from old lines
        /// </summary>
        /// <param name="color">The color the "message" will be printed out as in HB Output window</param>
        /// <param name="message"></param>
        /// <param name="args"></param>
        public static void Write(Color color, string message, params object[] args)
        {
            //if (LastWriteMessage == message)
            //    return;

            string sUniqueChar = (logLineNo++ & 1) == 0 ? "" : " ";
            System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B);

            if (GlobalSettings.Instance.LogLevel >= LogLevel.Normal)
                Logging.Write(newColor, "[IronBreaker]: " + message + sUniqueChar, args);
            else if (GlobalSettings.Instance.LogLevel == LogLevel.Quiet)
                Logging.WriteToFileSync(LogLevel.Normal, "[IronBreaker]: " + message + sUniqueChar, args);
            //LastWriteMessage = message;
        }

        #endregion

        #region [Method] - Pull Log

        public static void PullLog(string message, Color logColor, params object[] args)
        {
            if (message == LastInfoMsg)
                return;

            Write(logColor, "{0}", message, args);
            LastInfoMsg = message;
        }

        #endregion

        #region [Method] - Combat Log

        public static void CombatLog(string message, Color color, params object[] args)
        {
            if (message == LastCombatMsg)
                return;

            if (M.IS.EnableDebugLog)
            {
                if (CurrentTarget != null)
                    Write(color, "{0}" + string.Format($" on {CurrentTarget.SafeName}"), message, args);
                else
                    Write(color, "{0}" + " on [No Target!]", message, args);
            }
            else
                Write(color, "{0}", message, args);

            LastCombatMsg = message;
        }

        #endregion

        #region [Method] - Defensive Log

        public static void DefenseLog(string message, Color color, params object[] args)
        {
            if (message == LastDefenseLogMsg)
                return;

            System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B);
            Logging.Write(newColor, "[IronBreaker]: {0}", message);
            LastDefenseLogMsg = message;
        }

        #endregion

        #region [Method] - Debug Logging

        /// <summary>
        /// Writes a message to HB Output window if "DebugLog" is enabled in routine settings
        /// </summary>
        /// <param name="message">Message to write</param>
        public static void DebugWrite(string message)
        {
            if (LastDebugMsg == message)
                return;

            DebugWrite(Coloring.Debug, message);

            LastDebugMsg = message;
        }

        /// <summary>
        /// Writes a message to HB Output window if "DebugLog" is enabled in routine Settings
        /// </summary>
        /// <param name="message">Message to write</param>
        /// <param name="args">Replacement parameter values</param>
        public static void DebugWrite(string message, params object[] args)
        {
            if (LastDebugMsg == message)
                return;

            DebugWrite(Coloring.Debug, message, args);
            LastDebugMsg = message;
        }

        /// <summary>
        /// Writes a DEBUG message to HB Output window if "DebugLog" is enabled in routine settings
        /// </summary>
        /// <param name="color">Color of Debug Message in HB Output Window</param>
        /// <param name="message">Debug message to write</param>
        /// <param name="args">Replacement parameter values</param>
        public static void DebugWrite(Color color, string message, params object[] args)
        {
            if (LastDebugMsg == message)
                return;

            if (IronBreakerSettings.Instance.EnableDebugLog)
            {
                System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B);
                Logging.Write(newColor, "[IronBreaker::DEBUG] " + message, args);
                LastDebugMsg = message;
            }
        }

        #endregion

        #region [Method] - Diagnostic Logging

        /// <summary>
        /// Writes a message to HB Output window if "DebugLog" is enabled in routine settings
        /// </summary>
        /// <param name="message">Message to write</param>
        public static void DiagnosticLog(string message)
        {
            DiagnosticLog(Coloring.Debug, message);
        }

        /// <summary>
        /// Writes a message to HB Output window if "DebugLog" is enabled in routine settings
        /// </summary>
        /// <param name="message">Message to write</param>
        /// <param name="args">Parameter replacement values</param>
        public static void DiagnosticLog(string message, params object[] args)
        {
            DiagnosticLog(Coloring.Debug, message, args);
        }

        public static void DiagnosticLog(Color color, string message, params object[] args)
        {
            if (IronBreakerSettings.Instance.EnableDebugLog)
            {
                System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B);
                Logging.Write(newColor, "[IronBreaker:Diagnostic] " + message, args);
            }
        }

        #endregion

        #region [Method] - Info / Item Logging

        public static void UseItemLog(string message, Color color, params object[] args)
        {
            if (message == LastItemUseMsg)
                return;

            System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B);
            Logging.Write(newColor, "[IronBreaker]: {0}", message);
            LastItemUseMsg = message;
        }

        public static void InfoLog(string message, Color color, params object[] args)
        {
            if (message == LastInfoMsg)
                return;

            System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B);
            Logging.Write(newColor, "[IronBreaker]: {0}", message);
            LastInfoMsg = message;
        }

        #endregion
    }
}
