﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media;
using CommonBehaviors.Actions;
using IronBreaker.Core;
using IronBreaker.Core.Helpers;
using IronBreaker.Core.Managers;
using IronBreaker.Core.Utilities;
using IronBreaker.Settings;
using MahApps.Metro.Controls;
using Styx;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.CommonBot.Routines;
using Styx.TreeSharp;
using Styx.WoWInternals.WoWObjects;

#region Class Redundancy
using C = IronBreaker.Core.Helpers.Common;
using D = IronBreaker.Rotations.Death;
using U = IronBreaker.Core.Unit;
using T = IronBreaker.Core.Managers.TalentManagement;
#endregion

namespace IronBreaker
{
    public partial class Main : CombatRoutine
    {
        #region Declarations
        private static LocalPlayer Me => StyxWoW.Me;
        public static IronBreakerSettings IS = IronBreakerSettings.Instance;
        public static uint Latency { get; set; }
        private static readonly WaitTimer LatencyCheckTimer = new WaitTimer(TimeSpan.FromSeconds(5));
        public static readonly Version Version = new Version("1.0.8");
        public static string CrName => "IronBreaker";
        public override string Name => string.Format($"{CrName} {Version}");
        public override WoWClass Class => WoWClass.Warrior;
        public override bool WantButton => true;
        public bool Opened;
        #endregion

        public UserControl Control
        {
            get
            {
                using (var fs = new FileStream(@"Routines\IronBreaker\Interface\MainWindow.xaml", FileMode.Open))
                {
                    UserControl root = (UserControl)XamlReader.Load(fs);
                    root.DataContext = IronBreakerSettings.Instance;
                    return root;
                }
            }
        }

        public override void Initialize()
        {
            Update.CheckForUpdate();
            T.InitializeTalents();
            GetCurrentContext();
            IronBreakerBotEventInitialize();
            UpdateContexts();

            ContextChanged += (orig, ne) =>
            {
                Log.Write(Coloring.HighNormal, "Context Changed, re-creating behaviors");
                WriteContext();
                RebuildBehaviors();
            };

            IS.HotkeyInit();
            IS.ConditionInit();
            HotkeyManagement.RegisterHotkeys();
            BotEvents.OnBotStarted += BotEvents_OnBotStarted;
            BotEvents.OnBotStopped += BotEvents_OnBotStopped;
        }

        public override void OnButtonPress()
        {

            MetroWindow setForm = new MetroWindow()
            {
                Title = "[IronBreaker] Configuration",
                Content = Control,
                BorderThickness = new Thickness(0),
                WindowTitleBrush = Brushes.DimGray,
                TitleForeground = Brushes.MediumPurple,
                ShowCloseButton = true,
                ShowMinButton = false,
                ShowMaxRestoreButton = false,
                ShowIconOnTitleBar = false,
                SizeToContent = SizeToContent.WidthAndHeight,
                VerticalAlignment = VerticalAlignment.Stretch,
                HorizontalAlignment = HorizontalAlignment.Stretch,
                ResizeMode = ResizeMode.NoResize,
            };
            if (Opened == false)
            {
                setForm.Show();
                Opened = true;
            }
            setForm.Closed += setForm_Closed;
        }

        private void setForm_Closed(object sender, EventArgs e)
        {
            IS.ConditionInit();
            IS.Save();
            Log.Write(Coloring.Info, "***** Settings Saved *****");
            Opened = false;
        }

        private void BotEvents_OnBotStarted(EventArgs args)
        {
            if (IsBotInUse("Quest", "BGBuddy", "BG Bot", "DungeonBuddy", "Profession", "Gather"))
            {
                IS.AutoMove = true;
                IS.AutoFace = true;
                IS.AutoTarget = true;
            }

            Update.CheckForUpdate();
            T.RemoveTalentEvents();
            T.InitializeTalents();
            if (Me.Specialization == WoWSpec.WarriorArms)
                C.UpdateArtifactPowers();
            HotkeyManagement.RegisterHotkeys();
        }

        private void BotEvents_OnBotStopped(EventArgs args)
        {
            HotkeyManagement.RemoveHotkeys();
            T.RemoveTalentEvents();
        }

        public override void ShutDown()
        {
            HotkeyManagement.RemoveHotkeys();
        }

        public override void Pulse()
        {
            if (!StyxWoW.IsInGame || !StyxWoW.IsInWorld || Me == null || !Me.IsValid || !Me.IsAlive || Me.Mounted)
                return;
            if (!Me.Combat)
                return;

            if (LatencyCheckTimer.IsFinished)
            {
                Latency = StyxWoW.WoWClient.Latency;
                LatencyCheckTimer.Reset();
            }

            if (IS.IgnorePvP && U.PvPUnit(Me.CurrentTarget))
                Me.ClearTarget();

            C.Cache();
            U.EnemyAnnex(50f);

            if (!Me.IsCasting || !Me.IsChanneling)
            {
                if (GetTauntTarget())
                    return;
                if (GetInterruptableTargets())
                    return;
                if (GetValidTarget())
                    return;
            }

            UpdateContexts();
        }

        #region Behaviors

        public override Composite PullBehavior
        {
            get { return new ActionRunCoroutine(ctx => C.SpecSelectorPullBehavior()); }
        }
        public override Composite MoveToTargetBehavior
        {
            get { return new ActionRunCoroutine(ctx => Movement.MoveToTarget(Me.CurrentTarget)); }
        }
        public override Composite PreCombatBuffBehavior
        {
            get
            {
                return new ActionRunCoroutine(ctx => C.SpecSelectorPreCombatBuffBehavior());
            }
        }
        public override Composite CombatBuffBehavior
        {
            get
            {
                return new ActionRunCoroutine(ctx => C.SpecSelectorCombatBuffBehavior());
            }
        }
        public override bool NeedCombatBuffs => base.NeedCombatBuffs;
        public override Composite CombatBehavior
        {
            get
            {
                return new ActionRunCoroutine(ctx => C.SpecSelectorRotation());
            }
        }
        public override Composite RestBehavior
        {
            get
            {
                return new ActionRunCoroutine(ctx => C.RestSelector());
            }
        }

        public override bool NeedRest => Me.HealthPercent < IS.RestingValue;
        public override bool NeedDeath => Me.IsDead;
        public override Composite DeathBehavior { get { return new ActionRunCoroutine(ctx => D.OnDeath()); } }

        #endregion

        #region [Method] - Get Targets

        public bool GetInterruptableTargets()
        {
            if (IS.PummelHeal && !Spell.OnCooldown(SpellBook.Pummel))
            {
                var units = U.ActiveEnemies(Me.Location, 8f);
                var unit = units?.Where(u => u.IsCastingHealingSpell && u.CanInterruptCurrentSpellCast).OrderBy(d => d.Distance).FirstOrDefault();
                if (unit != null) { unit.Target(); return true; }
            }

            if (IS.PummelAny && !Spell.OnCooldown(SpellBook.Pummel))
            {
                var units = U.ActiveEnemies(Me.Location, 8f);
                var unit = units?.Where(u => (u.IsCasting || u.IsCastingHealingSpell) && u.CanInterruptCurrentSpellCast).OrderBy(d => d.Distance).FirstOrDefault();
                if (unit != null) { unit.Target(); return true; }
            }

            if (IS.StormBoltHeal && T.StormBolt && !Spell.OnCooldown(SpellBook.StormBolt))
            {
                var units = U.ActiveEnemies(Me.Location, 20f);
                var unit = units?.Where(u => u.IsCastingHealingSpell && u.CanInterruptCurrentSpellCast).OrderBy(d => d.Distance).FirstOrDefault();
                if (unit != null) { unit.Target(); return true; }
            }

            if (IS.StormBoltAny && T.StormBolt && !Spell.OnCooldown(SpellBook.StormBolt))
            {
                var units = U.ActiveEnemies(Me.Location, 20f);
                var unit = units?.Where(u => (u.IsCasting || u.IsCastingHealingSpell) && u.CanInterruptCurrentSpellCast).OrderBy(d => d.Distance).FirstOrDefault();
                if (unit != null) { unit.Target(); return true; }
            }
            return false;
        }

        public bool GetTauntTarget()
        {
            if (Me.Specialization == WoWSpec.WarriorProtection && IS.AutoTaunt)
            {
                U.TauntAnnex(50f);

                if (U.ActiveEnemiesToTaunt(Me.Location, 40f).Any())
                {
                    IEnumerable<WoWUnit> tauntTargets = U.ActiveEnemiesToTaunt(Me.Location, 40f);
                    var tauntTarget = tauntTargets?.OrderBy(u => u.Distance).FirstOrDefault();

                    if (tauntTarget != null)
                    {
                        if (IS.EnableDebugLog)
                            Log.PullLog(string.Format($"Taunting target: {tauntTarget.SafeName} at {tauntTarget.Distance:F0}yds HP: {tauntTarget.HealthPercent:F0}%"), Coloring.Tank);

                        tauntTarget.Target();
                        return true;
                    }
                }
            }
            return false;
        }

        public bool GetValidTarget()
        {
            if (!Me.CurrentTarget.IsValidCombatUnit() || !U.IsValidUnit(Me.CurrentTarget))
            {
                var targets = U.ActiveEnemies(Me.Location, 30f).OrderBy(u => u.Distance).ThenBy(u => u.HealthPercent);
                {
                    var target = targets.FirstOrDefault();
                    if (target == null) return false;
                    target.Target();
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Capability Flags

        public static bool CapabilityActive(CapabilityFlags flag)
        {
            return RoutineManager.GetCapabilityState(flag) == CapabilityState.DontCare;
        }

        public override CapabilityFlags SupportedCapabilities
        {
            get
            {
                var capabilities = (int)CapabilityFlags.Aoe;
                if (IS.AutoMove) { capabilities += (int)CapabilityFlags.Movement; }
                if (IS.AutoTarget) { capabilities += (int)CapabilityFlags.Targeting; }
                if (IS.AutoFace) { capabilities += (int)CapabilityFlags.Facing; }
                //if (IS.MultiPull) { capabilities += (int)CapabilityFlags.MultiMobPull; }

                if (Me.Specialization == WoWSpec.WarriorArms)
                {
                    if (IS.EnableArmsDefensive) { capabilities += (int)CapabilityFlags.DefensiveCooldowns; }
                    if (IS.EnableArmsOffensive) { capabilities += (int)CapabilityFlags.OffensiveCooldowns; }
                    if (IS.ArmsInterruptAny || IS.ArmsInterruptHeal) { capabilities += (int)CapabilityFlags.Interrupting; }
                    if (IS.AEnableCharge || IS.AEnableHeroicLeap || IS.AEnableHeroicThrow) { capabilities += (int)CapabilityFlags.GapCloser; }
                }

                if (Me.Specialization == WoWSpec.WarriorFury)
                {
                    if (IS.EnableFuryDefensive) { capabilities += (int)CapabilityFlags.DefensiveCooldowns; }
                    if (IS.EnableFuryOffensive) { capabilities += (int)CapabilityFlags.OffensiveCooldowns; }
                    if (IS.FuryInterruptAny || IS.FuryInterruptHeal) { capabilities += (int)CapabilityFlags.Interrupting; }
                    if (IS.FEnableCharge || IS.FEnableHeroicLeap || IS.FEnableHeroicThrow) { capabilities += (int)CapabilityFlags.GapCloser; }
                }

                if (Me.Specialization == WoWSpec.WarriorProtection)
                {
                    if (IS.EnableProtDefensive) { capabilities += (int)CapabilityFlags.DefensiveCooldowns; }
                    if (IS.EnableProtOffensive) { capabilities += (int)CapabilityFlags.OffensiveCooldowns; }
                    if (IS.ProtInterruptAny || IS.ProtInterruptHeal) { capabilities += (int)CapabilityFlags.Interrupting; }
                    if (IS.AutoTaunt) { capabilities += (int)CapabilityFlags.Taunting; }
                    if (IS.PEnableHeroicLeap || IS.EnableIntercept || IS.PEnableHeroicThrow) { capabilities += (int)CapabilityFlags.GapCloser; }
                }
                return (CapabilityFlags)capabilities;
            }
        }

        #endregion
    }
}
