﻿using System.Linq;
using System.Threading.Tasks;
using Buddy.Coroutines;
using IronBreaker.Core;
using IronBreaker.Core.Helpers;
using IronBreaker.Core.Managers;
using IronBreaker.Core.Utilities;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

#region Class Redundancy
using C = IronBreaker.Core.Utilities.Coloring;
using CM = IronBreaker.Core.Helpers.Common;
using M = IronBreaker.Main;
using SB = IronBreaker.Core.Helpers.SpellBook;
using S = IronBreaker.Core.Spell;
using U = IronBreaker.Core.Unit;
using T = IronBreaker.Core.Managers.TalentManagement;
#endregion

namespace IronBreaker.Rotations
{
    internal class Arms
    {
        #region Declarations
        private static LocalPlayer Me => StyxWoW.Me;
        private static WoWUnit CurrentTarget => StyxWoW.Me.CurrentTarget;
        //private static bool HasShatteredDefenses => CM.CurrentArtifactRank(1137) != 0;
        //private static bool AppliedShatteredDefenses => HasShatteredDefenses && Me.HasAura(SB.auraShatteredDefenses);
        //private static bool UnAppliedShatteredDefenses => HasShatteredDefenses && !Me.HasAura(SB.auraShatteredDefenses);
        private static uint FrStacks => U.AuraStackCount(Me, SB.auraArmsFocusedRage, true);
        private static bool DeadlyCalmBc => T.DeadlyCalm && Me.HasAura(SB.auraBattleCry);
        #endregion

        #region [Method] - Rotation Selector

        public static async Task<bool> RotationSelector()
        {
            if (!Me.IsAlive || Me.OnTaxi || Me.IsCasting || Me.IsChanneling)
                return true;

            if (await Pull()) { return true; }
            await Interrupt();
            await Movement.FaceCurrentTarget(CurrentTarget);
            await Movement.MoveToTarget(CurrentTarget);

            if (await S.Gcd(SB.ArmsStance, C.Combat,
                M.IS.EnableDefensiveStance
                && Me.HasAura(SB.auraDefensiveStance)
                && M.IS.DefensiveStanceValue > 0
                && Me.HealthPercent > M.IS.DefensiveStanceValue + 10,
                "Switching Stances (Arms)"))
            { return true; }

            if (CurrentTarget.IsValidCombatUnit())
            {
                if (await S.Cast(SB.VictoryRush, C.Combat,
                    CurrentTarget.IsWithinMeleeRangeOf(Me)
                    && Me.HasAura(SB.auraVictorious)
                    && CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "Victory Rush Proc"))
                { return true; }

                if (await S.Cast(SB.Rend, C.Combat, 
                    T.Rend 
                    && (CM.CurrentRage >= 15 || DeadlyCalmBc) 
                    && !CurrentTarget.HasAura(SB.auraRend)
                    && CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[ST] Applying Rend"))
                { return true; }

                if (await S.Cast(SB.ArmsExecute, C.Combat,
                    M.IS.EnableStoneHeart
                    && Me.HasAnyTempAura("Stone Heart")
                    && CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[ST] Execute (Stone Heart Proc)"))
                { return true; }

                if (await S.Cast(SB.ArmsExecute, C.Combat,
                    Me.HasAura(SB.auraShatteredDefenses) || Me.HasAura(SB.auraBattleCry)
                    && CurrentTarget.HealthPercent < 20
                    && ((!T.Dauntless && CM.CurrentRage >= 22) || (T.Dauntless && CM.CurrentRage >= 18) || DeadlyCalmBc)
                    && CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[FR:ST] Execute (SD Up)"))
                { return true; }

                if (await OnDefense()) { return true; }
                if (BurstCondition()) { if (await Burst()) return true; }

                if ((M.IS.AutoAoE && M.IS.AoEValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.AoEValue)
                    || (!M.IS.AutoAoE && U.ActiveEnemies(Me.Location, 8f).Count() >= 3))
                {
                    if (await AoE())
                        return true;
                }

                if (T.FocusedRage && M.IS.AEnableFocusedRage) { if (await FocusedRageSt()) return true; }
                if (T.MortalCombo) { if (await MortalComboSt()) return true; }
            }
            return false;
        }

        #endregion

        #region [Method] - PreCombat Buffing Behavior

        public static async Task<bool> PreCombatBuffing()
        {
            if (Me.Combat || !Me.IsAlive || Me.OnTaxi)
                return true;

            await Coroutine.Yield();
            return false;
        }

        #endregion

        #region [Method] Pulling / Gap Closer Behavior

        public static async Task<bool> Pull()
        {
            if (CurrentTarget.IsValidCombatUnit() )
            {
                await Movement.FaceCurrentTarget(CurrentTarget);
                await Movement.MoveToTarget(CurrentTarget);

                if (M.IS.AutoPull)
                {
                    if (await S.Cast(SB.HeroicThrow, C.Combat,
                        CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.Distance <= 30
                        && CurrentTarget.Distance >= 8
                        && !Me.HasAnyTempAura("Charge")
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Heroic Throw"))
                    { return true; }

                    if (await S.Cast(SB.Charge, C.Combat,
                        CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.Distance <= 25
                        && CurrentTarget.Distance >= 8
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Charge"))
                    { await Coroutine.Sleep(500); return true; }

                    if (await S.CastGround(SB.HeroicLeap, C.Combat,
                        CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.InLineOfSight
                        && CurrentTarget.Distance <= 30
                        && CurrentTarget.Distance >= 8
                        && !Me.HasAnyTempAura("Charge")
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Heroic Leap"))
                    { await Coroutine.Sleep(800); return true; }
                }

                if (await S.Cast(SB.HeroicThrow, C.Combat,
                    M.IS.AEnableHeroicThrow
                    && CurrentTarget.InLineOfSpellSight
                    && !Me.HasAnyTempAura("Charge")
                    && CurrentTarget.Distance <= 30
                    && CurrentTarget.Distance >= 8
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Heroic Throw"))
                { return true; }

                if (await S.Cast(SB.Charge, C.Combat,
                    M.IS.AEnableCharge
                    && CurrentTarget.InLineOfSpellSight
                    && CurrentTarget.Distance <= 25
                    && CurrentTarget.Distance >= 8
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Charge"))
                { await Coroutine.Sleep(500); return true; }

                if (await S.CastGround(SB.HeroicLeap, C.Combat,
                    M.IS.AEnableHeroicLeap
                    && CurrentTarget.InLineOfSpellSight
                    && CurrentTarget.InLineOfSight
                    && !Me.HasAnyTempAura("Charge")
                    && CurrentTarget.Distance <= 30
                    && CurrentTarget.Distance >= 8
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Heroic Leap"))
                { await Coroutine.Sleep(800); return true; }
            }
            return false;
        }

        #endregion

        #region [Method] - Combat Buffing Behavior

        public static async Task<bool> CombatBuffing()
        {
            if (!Me.IsAlive || !Me.GotTarget || !CurrentTarget.IsAlive || Me.OnTaxi)
                return true;

            if (!Me.IsAutoAttacking && CurrentTarget.IsValidCombatUnit())
            {
                Lua.DoString("StartAttack()");
                return true;
            }
            await Coroutine.Yield();
            return false;
        }

        #endregion

        #region [Method] - Defensive Behavior

        public static async Task<bool> OnDefense()
        {
            #region Talents

            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.HpBelow
                && M.IS.ShockwaveValue > 0
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10,
                "[Defense] Shockwave (HP)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Combat,
                M.IS.EnableStormBolt
                && T.StormBolt
                && M.IS.StormBolt == SingleStunType.HpBelow
                && M.IS.StormBoltValue > 0
                && Me.HealthPercent < M.IS.StormBoltValue
                && CurrentTarget.Distance <= 20,
                "[Defense] Storm Bolt (HP)"))
            { return true; }

            #endregion

            #region Defensive Stance
            if (await S.Gcd(SB.DefensiveStance, C.Combat,
                M.IS.EnableDefensiveStance
                && T.DefensiveStance
                && ((M.IS.DefensiveStance == DefenseType.HpBelow && M.IS.DefensiveStanceValue > 0 && Me.HealthPercent < M.IS.DefensiveStanceValue)
                    || (M.IS.DefensiveStance == DefenseType.TargetCount && M.IS.DefensiveStanceValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.DefensiveStanceValue)),
                "[Defense] Defensive Stance"))
            { return true; }
            #endregion

            #region Die by the Sword
            if (await S.Gcd(SB.DieByTheSword, C.Combat,
                M.IS.EnableDieByTheSword
                && M.IS.DieByTheSwordValue > 0
                && Me.HealthPercent < M.IS.DieByTheSwordValue
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Defense] Die by the Sword"))
            { return true; }
            #endregion

            #region Commanding Shout
            if (await S.Gcd(SB.CommandingShout, C.Combat,
                M.IS.AEnableCommandingShout
                && M.IS.ACommandingShoutValue > 0
                && Me.HealthPercent < M.IS.ACommandingShoutValue
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Commanding Shout"))
            { return true; }
            #endregion

            #region Berserker Rage

            if (await S.Gcd(SB.BerserkerRage, C.Combat,
                M.IS.EnableBerserkerRage
                && ((M.IS.BerserkerRage == ClassType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite()) || M.IS.BerserkerRage == ClassType.OnCooldown)
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (Me.HasAuraWithEffect(WoWApplyAuraType.ModFear) || Me.HasAnyTempAura("Sap") || Me.HasAnyTempAura("Repentance") || Me.HasAnyTempAura("Gouge")),
                "[Defense] Berserker Rage"))
            { return true; }

            #endregion

            if (await S.Gcd(SB.IntimidatingShout, C.Combat,
                M.IS.AEnableIntimidatingShout
                && ((M.IS.AIntimidatingShout == CrowdControlType.HpBelow && M.IS.AIntimidatingShoutValue > 0 && Me.HealthPercent < M.IS.AIntimidatingShoutValue)
                    || (M.IS.AIntimidatingShout == CrowdControlType.TargetCount && M.IS.AIntimidatingShoutValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.AIntimidatingShoutValue))
                && U.ActiveEnemies(Me.Location, 8f).Any(),
                "[Defense] Intimidating Shout"))
            { await Coroutine.Sleep(2000); return true; }

            return false;
        }

        #endregion

        #region [Method] - Cooldowns

        public static async Task<bool> Cooldowns()
        {
            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.OnCooldown
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10,
                "[CD] Shockwave"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Combat,
                M.IS.EnableStormBolt
                && T.StormBolt
                && M.IS.StormBolt == SingleStunType.OnCooldown
                && CurrentTarget.Distance <= 20,
                "[CD] Storm Bolt"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Single Target Behaviors

        public static async Task<bool> FocusedRageSt()
        {
            if (await S.Cast(SB.ArmsExecute, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && S.OnCooldown(SB.ColossusSmash)
                && CurrentTarget.HealthPercent < 20
                && ((!T.Dauntless && CM.CurrentRage >= 22) || (T.Dauntless && CM.CurrentRage >= 18) || DeadlyCalmBc),
                "[FR:ST] Execute (No SD, CS on CD)"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                T.Avatar
                && M.IS.ABattleCryAuto
                && M.IS.AEnableBattleCry
                && CurrentTarget.HasAura(SB.auraColossusSmash)
                && (CurrentTarget.HealthPercent >= 20 || (CurrentTarget.HealthPercent < 20 && FrStacks >= 2))
                && (!S.OnCooldown(SB.Avatar) || FrStacks >= 2 || CM.CurrentRage < 25)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                !T.Avatar
                && M.IS.AEnableBattleCry
                && M.IS.ABattleCryAuto
                && CurrentTarget.HasAura(SB.auraColossusSmash)
                && (CurrentTarget.HealthPercent >= 20 || (CurrentTarget.HealthPercent < 20 && FrStacks >= 2))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                M.IS.AEnableBattleCry
                && M.IS.ABattleCryCd
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.Avatar, C.Combat,
                T.Avatar
                && (M.IS.Avatar == ClassType.OnCooldown || (M.IS.Avatar == ClassType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite()))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Avatar"))
            { return true; }

            if (await S.Cast(SB.ColossusSmash, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Colossus Smash (Initial)"))
            { return true; }

            if (await S.Cast(SB.ArmsFocusedRage, C.Combat, ((!T.Dauntless && CM.CurrentRage >= 15) 
                    || (T.Dauntless && CM.CurrentRage >= 12) || DeadlyCalmBc)
                && !Me.HasAura(SB.auraArmsFocusedRage)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Focused Rage (Initial)"))
            { return true; }

            if (await S.Cast(SB.ArmsFocusedRage, C.Combat,
                Me.HasAura(SB.auraArmsFocusedRage)
                && ((!T.Dauntless && CM.CurrentRage >= 40 && S.OnCooldown(SB.MortalStrike)) || (T.Dauntless && CM.CurrentRage >= 36 && S.OnCooldown(SB.MortalStrike))
                    || (!T.Dauntless && !S.OnCooldown(SB.MortalStrike) && CM.CurrentRage >= 15) 
                    || (!S.OnCooldown(SB.MortalStrike) && T.Dauntless && CM.CurrentRage >= 12) 
                    || DeadlyCalmBc || (CM.DrainedRage <= 25 && FrStacks < 3))
                && FrStacks < 3
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] FR (Stacking)"))
            { return true; }

            if (await S.Cast(SB.Bladestorm, C.Combat,
                BladestormSt()
                && !T.Ravager
                && CM.CurrentRage < 32
                && !DeadlyCalmBc
                && S.OnCooldown(SB.ColossusSmash)
                && S.OnCooldown(SB.MortalStrike)
                && (T.Overpower && !Me.HasAura(SB.auraOverpower) || !T.Overpower)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Bladestorm (All on CD, no rage for Slam)"))
            { return true; }

            if (await S.Cast(SB.ArmsRavager, C.Combat,
                M.IS.EnableRavager
                && !DeadlyCalmBc
                && T.Ravager
                && CM.CurrentRage < 32 && S.OnCooldown(SB.ColossusSmash) && S.OnCooldown(SB.MortalStrike)
                && (T.Overpower && !Me.HasAura(SB.auraOverpower) || !T.Overpower)
                && M.IS.Ravager == ArmsType.OnCooldown
                && CurrentTarget.Distance <= 30,
                "[FR:ST] Ravager (All on CD, no rage for Slam)"))
            { return true; }

            if (await S.Cast(SB.MortalStrike, C.Combat,
                Me.HasAura(SB.auraArmsFocusedRage)
                && (Me.HasAura(SB.auraShatteredDefenses) || FrStacks >= 2)
                && (!T.Dauntless && CM.CurrentRage >= 20  || (T.Dauntless && CM.CurrentRage >= 16) || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Mortal Strike w/ FR Stacks"))
            { return true; }

            if (await S.Cast(SB.Warbreaker, C.Combat,
                M.IS.EnableWarbreaker
                && M.IS.WarbreakerAuto
                && !Me.HasAura(SB.auraShatteredDefenses)
                && S.OnCooldown(SB.ColossusSmash)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Warbreaker (No SD, CS on CD)"))
            { return true; }

            if (await S.Cast(SB.Overpower, C.Combat,
                T.Overpower
                && Me.HasAura(SB.auraOverpower)
                && Me.HasAura(SB.auraShatteredDefenses) || S.OnCooldown(SB.ColossusSmash)
                && (CM.CurrentRage >= 10 || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Overpower"))
            { return true; }

            if (await S.Cast(SB.Slam, C.Combat,
                !T.FervorOfBattle
                && S.OnCooldown(SB.ColossusSmash)
                && S.OnCooldown(SB.MortalStrike)
                && ((T.Overpower && !Me.HasAura(SB.auraOverpower)) || !T.Overpower)
                && ((!T.Dauntless && CM.CurrentRage > 44) || (T.Dauntless && CM.CurrentRage > 35) || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Slam (CS, MS on CD, no OP Proc)"))
            { return true; }

           if (await S.Cast(SB.ArmsWhirlwind, C.Combat,
                T.FervorOfBattle
                && ((!T.Dauntless && CM.CurrentRage >= 44) || (T.Dauntless && CM.CurrentRage >= 35) || DeadlyCalmBc) 
                && S.OnCooldown(SB.ColossusSmash) 
                && S.OnCooldown(SB.MortalStrike)
                && ((T.Overpower && !Me.HasAura(SB.auraOverpower)) || !T.Overpower)
                && CurrentTarget.IsWithinMeleeRangeOf(Me), 
                "[ST] Whirlwind (Fervor of Battle)"))
            { return true; }

            return false;
        }

        public static async Task<bool> MortalComboSt()
        {
            if (await S.Cast(SB.ArmsExecute, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && S.OnCooldown(SB.ColossusSmash)
                && CurrentTarget.HealthPercent < 20
                && ((!T.Dauntless && CM.CurrentRage >= 22) || (T.Dauntless && CM.CurrentRage >= 18) || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Execute (No SD, CS on CD)"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                T.Avatar
                && CurrentTarget.HasAura(SB.auraColossusSmash)
                && (!S.OnCooldown(SB.Avatar) || CM.CurrentRage < 25)
                && M.IS.AEnableBattleCry
                && M.IS.ABattleCryAuto
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                !T.Avatar
                && CurrentTarget.HasAura(SB.auraColossusSmash)
                && (!S.OnCooldown(SB.Avatar) || CM.CurrentRage < 30)
                && M.IS.AEnableBattleCry
                && M.IS.ABattleCryAuto
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                M.IS.AEnableBattleCry
                && M.IS.ABattleCryCd
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.Avatar, C.Combat,
                T.Avatar
                && (M.IS.Avatar == ClassType.OnCooldown || (M.IS.Avatar == ClassType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite()))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[FR:ST] Avatar"))
            { return true; }

            if (await S.Cast(SB.ColossusSmash, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Colossus Smash (Initial)"))
            { return true; }

            if (await S.Cast(SB.Bladestorm, C.Combat,
                BladestormSt()
                && !T.Ravager
                && CM.CurrentRage < 32
                && S.OnCooldown(SB.ColossusSmash)
                && S.OnCooldown(SB.MortalStrike)
                && (T.Overpower && !Me.HasAura(SB.auraOverpower) || !T.Overpower)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Bladestorm (All on CD, no rage for Slam)"))
            { return true; }

            if (await S.Cast(SB.ArmsRavager, C.Combat,
                M.IS.EnableRavager
                && T.Ravager
                && CM.CurrentRage < 32 && S.OnCooldown(SB.ColossusSmash) && S.OnCooldown(SB.MortalStrike)
                && (T.Overpower && !Me.HasAura(SB.auraOverpower) || !T.Overpower)
                && M.IS.Ravager == ArmsType.OnCooldown
                && CurrentTarget.Distance <= 30,
                "[MC:ST] Ravager (All on CD, no rage for Slam)"))
            { return true; }

            if (await S.Cast(SB.MortalStrike, C.Combat,               
                ((Me.HasAura(SB.auraShatteredDefenses) && S.GetCharges(SB.MortalStrike) > 0) 
                    || S.GetCharges(SB.MortalStrike) > 1 
                    || (S.OnCooldown(SB.ColossusSmash) && S.OnCooldown(SB.Warbreaker) && S.GetCharges(SB.MortalStrike) > 0))
                && ((!T.Dauntless && CM.CurrentRage >= 20) || (T.Dauntless && CM.CurrentRage >= 16) || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Mortal Strike"))
            { return true; }

            if (await S.Cast(SB.Warbreaker, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && S.OnCooldown(SB.ColossusSmash)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Warbreaker (No SD, CS on CD)"))
            { return true; }

            if (await S.Cast(SB.Overpower, C.Combat,
                T.Overpower
                && Me.HasAura(SB.auraOverpower)
                && Me.HasAura(SB.auraShatteredDefenses) || S.OnCooldown(SB.ColossusSmash)
                && (CM.CurrentRage >= 10 || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Overpower"))
            { return true; }

            if (await S.Cast(SB.Slam, C.Combat,
                !T.FervorOfBattle
                && S.OnCooldown(SB.ColossusSmash)
                && S.GetCharges(SB.MortalStrike) < 1
                && ((T.Overpower && !Me.HasAura(SB.auraOverpower)) || !T.Overpower)
                && ((!T.Dauntless && CM.CurrentRage > 30) || (T.Dauntless && CM.CurrentRage > 26) || DeadlyCalmBc)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[MC:ST] Slam (CS, MS on CD, no OP Proc)"))
            { return true; }

            if (await S.Cast(SB.ArmsWhirlwind, C.Combat,
                 T.FervorOfBattle
                 && ((!T.Dauntless && CM.CurrentRage >= 30) || (T.Dauntless && CM.CurrentRage > 26) || DeadlyCalmBc)
                 && S.OnCooldown(SB.ColossusSmash)
                 && S.OnCooldown(SB.MortalStrike)
                 && ((T.Overpower && !Me.HasAura(SB.auraOverpower)) || !T.Overpower)
                 && CurrentTarget.IsWithinMeleeRangeOf(Me),
                 "[MC:ST] Whirlwind (Fervor of Battle)"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - AOE Behavior

        public static async Task<bool> AoE()
        {
            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.TargetCount
                && M.IS.ShockwaveValue > 0
                && U.ActiveEnemies(CurrentTarget.Location, 8f).Count() >= M.IS.ShockwaveValue
                && Me.IsSafelyFacing(CurrentTarget),
                "[AOE] Shockwave"))
            { return true; }

            if (await S.Gcd(SB.Bladestorm, C.Combat,
                M.IS.AEnableBladeStorm
                && !T.Ravager
                && M.IS.ABladestorm == ArmsType.TargetCount
                && M.IS.ABladestormValue > 0
                && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.ABladestormValue,
                "[AOE] Bladestorm"))
            { return true; }

            if (await S.Cast(SB.ArmsRavager, C.Combat,
                M.IS.EnableRavager
                && T.Ravager
                && M.IS.Ravager == ArmsType.TargetCount
                && M.IS.RavagerValue > 0
                && CurrentTarget.Distance <= 8
                && U.ActiveEnemies(CurrentTarget.Location, 8f).Count() >= M.IS.RavagerValue,
                "[AOE] Ravager"))
            { return true; }

            if (await S.Cast(SB.Cleave, C.Combat,
                (!T.Dauntless && CM.CurrentRage >= 10) || (T.Dauntless && CM.CurrentRage >= 8) || DeadlyCalmBc
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Cleave"))
            { return true; }

            if (await S.Cast(SB.Warbreaker, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && S.OnCooldown(SB.ColossusSmash)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Warbreaker"))
            { return true; }

            if (await S.Cast(SB.BattleCry, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.ArmsWhirlwind, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && ((!T.Dauntless && CM.CurrentRage >= 25) || (T.Dauntless && CM.CurrentRage >= 20) || DeadlyCalmBc),
                "[AOE] Whirlwind"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Burst Behavior

        public static async Task<bool> Burst()
        {
            await S.Gcd(SB.BerserkerRage, C.Combat,
                M.IS.EnableBerserkerRage
                && M.IS.BerserkerRage == ClassType.OnBurst
                && (Me.HasAuraWithEffect(WoWApplyAuraType.ModFear) || Me.HasAnyTempAura("Sap") || Me.HasAnyTempAura("Repentance") || Me.HasAnyTempAura("Gouge"))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Berserker Rage");

            await S.Gcd(SB.Warbreaker, C.Combat,
                M.IS.EnableWarbreaker
                && M.IS.WarbreakerAuto
                && !Me.HasAura(SB.auraShatteredDefenses)
                && S.OnCooldown(SB.ColossusSmash)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Warbreaker");

            await S.Cast(SB.ColossusSmash, C.Combat,
                !Me.HasAura(SB.auraShatteredDefenses)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Colossus Smash");

            await S.Gcd(SB.BattleCry, C.Combat,
                M.IS.AEnableBattleCry
                && M.IS.ABattleCryAuto
                && !S.OnCooldown(SB.Avatar)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Battle Cry");

            await S.Gcd(SB.Avatar, C.Combat,
                M.IS.EnableAvatar
                && T.Avatar
                && (M.IS.ABattleCryAuto || M.IS.ABattleCryCd)
                && Me.HasAura(SB.auraBattleCry)
                && M.IS.Avatar == ClassType.OnBurst
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Avatar");

            //await S.Cast(SB.MortalStrike, C.Combat,
            //    T.MortalCombo
            //    && Me.HasAura(SB.auraShatteredDefenses)
            //    && ((!T.Dauntless && CM.CurrentRage >= 20) || T.Dauntless && CM.CurrentRage >= 16)
            //    && CurrentTarget.IsWithinMeleeRangeOf(Me),
            //    "[Burst] Mortal Strike (SD Applied)");

            //await S.Cast(SB.MortalStrike, C.Combat,
            //    T.MortalCombo
            //    && !Me.HasAura(SB.auraShatteredDefenses)
            //    && S.GetCharges(SB.MortalStrike) > 1
            //    && CurrentTarget.IsWithinMeleeRangeOf(Me),
            //    "[Burst] Mortal Strike (SD Unapplied)");

            //await S.Cast(SB.MortalStrike, C.Combat,
            //    T.FocusedRage
            //    && FrStacks >= 2
            //    && Me.HasAura(SB.auraShatteredDefenses)
            //    && CurrentTarget.IsWithinMeleeRangeOf(Me),
            //    "[Burst] Mortal Strike w/ FR Stacks");

            if (await S.Cast(SB.Bladestorm, C.Combat,
                M.IS.AEnableBladeStorm
                && !T.Ravager
                && M.IS.ABladestorm == ArmsType.OnBurst
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Bladestorm"))
            { return true; }

            if (await S.Cast(SB.ArmsRavager, C.Combat,
                M.IS.EnableRavager
                && T.Ravager
                && M.IS.Ravager == ArmsType.OnBurst
                && CurrentTarget.Distance <= 30,
                "[Burst] Ravager"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Interrupt Behavior

        public static async Task<bool> Interrupt()
        {
            if (!M.IS.AllowArmInterrupts)
                return false;

            #region Interrupt Heal Cast

            if (await S.Cast(SB.Pummel, C.Interrupt,
                M.IS.PummelHeal
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Pummel (Heal)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Interrupt,
                M.IS.StormBoltHeal
                && T.StormBolt
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.Distance <= 20
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Storm Bolt (Heal)"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Interrupt,
                M.IS.ShockwaveHeal
                && T.Shockwave
                && CurrentTarget.IsValidCombatUnit()
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Shockwave (Heal)"))
            { return true; }

            if (await S.Gcd(SB.IntimidatingShout, C.Interrupt,
                M.IS.ArmsIntimidatingShoutHeal
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Intimidating Shout (Heal)"))
            { return true; }

            #endregion

            #region Interrupt Any Cast

            if (await S.Cast(SB.Pummel, C.Interrupt,
                M.IS.PummelAny
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Pummel (Any)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Interrupt,
                M.IS.StormBoltAny
                && T.StormBolt
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.Distance <= 20
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Storm Bolt (Any)"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Interrupt,
                M.IS.ShockwaveAny
                && T.Shockwave
                && CurrentTarget.IsValidCombatUnit()
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Shockwave (Any)"))
            { return true; }

            if (await S.Gcd(SB.IntimidatingShout, C.Interrupt,
                M.IS.ArmsIntimidatingShoutAny
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Intimidating Shout (Any)"))
            { return true; }

            #endregion

            return false;
        }

        #endregion

        #region [Method] - Casting Targets

        private static WoWUnit GetTargetsHealing(double range)
        {
            var units = U.ActiveEnemies(Me.Location, range);
            var target =
                units?.Where(u => u.IsCastingHealingSpell && u.CanInterruptCurrentSpellCast)
                    .OrderBy(d => d.Distance)
                    .FirstOrDefault();

            return target;
        }

        private static WoWUnit GetTargetCasting(double range)
        {
            var units = U.ActiveEnemies(Me.Location, range);

            var target =
                units?.Where(u => (u.IsCasting || u.IsCastingHealingSpell) && u.CanInterruptCurrentSpellCast)
                    .OrderBy(d => d.Distance)
                    .FirstOrDefault();
            return target;
        }

        #endregion

        #region [Method] - Conditions

        private static bool BladestormSt()
        {
            return M.IS.AEnableBladeStorm && M.IS.ABladestorm == ArmsType.OnCooldown;
        }

        private static bool BurstCondition()
        {
            return (M.IS.ArmsBurst == BurstType.HpBelow && M.IS.ArmsBurstValue > 0 && Me.HealthPercent < M.IS.ArmsBurstValue)
                    || (M.IS.ArmsBurst == BurstType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite())
                    || (M.IS.ArmsBurst == BurstType.TargetCount && M.IS.ArmsBurstValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.ArmsBurstValue)
                    || (M.IS.ArmsBurst == BurstType.TargetHpBelow && M.IS.ArmsBurstValue > 0 && CurrentTarget.HealthPercent < M.IS.ArmsBurstValue)
                    || M.IS.ArmsBurst == BurstType.OnCooldown
                    || (M.IS.ArmsBurst == BurstType.BurstKeys && HotkeyManagement.BurstOn) && (T.Avatar && !S.OnCooldown(SB.Avatar) || !T.Avatar) && !S.OnCooldown(SB.BattleCry) && !S.OnCooldown(SB.ColossusSmash) && S.GetCharges(SB.MortalStrike) > 0;
        }

        #endregion
    }
}
