﻿using System.Linq;
using System.Threading.Tasks;
using Buddy.Coroutines;
using IronBreaker.Core;
using IronBreaker.Core.Helpers;
using IronBreaker.Core.Managers;
using IronBreaker.Core.Utilities;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

#region Class Redundancy
using C = IronBreaker.Core.Utilities.Coloring;
using CM = IronBreaker.Core.Helpers.Common;
using M = IronBreaker.Main;
using SB = IronBreaker.Core.Helpers.SpellBook;
using S = IronBreaker.Core.Spell;
using U = IronBreaker.Core.Unit;
using T = IronBreaker.Core.Managers.TalentManagement;
#endregion

namespace IronBreaker.Rotations
{
    internal class Fury
    {
        #region Declarations
        private static LocalPlayer Me => StyxWoW.Me;
        private static WoWUnit CurrentTarget => StyxWoW.Me.CurrentTarget;
        private static bool _ragingBlowCast;
        #endregion

        #region [Method] - Rotation Selector

        public static async Task<bool> RotationSelector()
        {
            if (!Me.IsAlive || Me.OnTaxi || Me.IsCasting || Me.IsChanneling)
                return true;

            if (await Pull()) { return true; }
            await Interrupt();
            await Movement.FaceCurrentTarget(CurrentTarget);
            await Movement.MoveToTarget(CurrentTarget);

            if (CurrentTarget.IsValidCombatUnit())
            {
                await S.Cast(SB.FuryExecute, C.Combat, M.IS.EnableStoneHeart && Me.HasAnyTempAura("Stone Heart") && CurrentTarget.IsWithinMeleeRangeOf(Me), "[Proc] Execute");
                await S.Cast(SB.FuryExecute, C.Combat, CurrentTarget.HealthPercent < 20 && Me.HasAura(SB.auraEnrage) && CM.CurrentRage >= 25 && CurrentTarget.IsWithinMeleeRangeOf(Me), "[Proc] Enrage (Enrage Applied)");
                await S.Cast(SB.Rampage, C.Combat, T.Massacre && Me.HasAura(SB.auraMassacre) && CurrentTarget.IsWithinMeleeRangeOf(Me), "[Proc] Rampage (Massacre)");
                await S.Cast(SB.FuryWhirlwind, C.Combat, T.WreckingBall && Me.HasAura(SB.auraWreckingBall) && CurrentTarget.IsWithinMeleeRangeOf(Me), "[Proc] Whirlwind (Wrecking Ball)");

                if (await OnDefense()) { return true; }
                if (await GapCloser()) { return true; }         
                if (BurstCondition()) { if (await Burst()) return true; }
                if (await Cooldowns()) { return true; }
                if ((M.IS.AutoAoE && M.IS.AoEValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.AoEValue) || (!M.IS.AutoAoE && U.ActiveEnemies(Me.Location, 8f).Count() >= 3))
                {
                    if (await AoE()) { return true; }
                }
                return await SingleTarget();
            }
            return false;
        }

        #endregion

        #region [Method] - PreCombat Buffing Behavior

        public static async Task<bool> PreCombatBuffing()
        {
            if (Me.Combat || !Me.IsAlive || Me.OnTaxi)
                return true;

            await Coroutine.Yield();
            return false;
        }

        #endregion

        #region [Method] Pulling / Gap Closer Behavior

        public static async Task<bool> GapCloser()
        {
            if (await S.Cast(SB.Charge, C.Combat,
                !Me.IsWithinMeleeRangeOf(CurrentTarget) && CurrentTarget.Distance >= 8 && CurrentTarget.Distance <= 30 && CurrentTarget.InLineOfSight && CurrentTarget.InLineOfSpellSight, "[GapClose] Charge")) { return true; }

            if (await S.Cast(SB.HeroicLeap, C.Combat,
                !Me.IsWithinMeleeRangeOf(CurrentTarget) && CurrentTarget.Distance >= 15 && CurrentTarget.Distance <= 30 && CurrentTarget.InLineOfSight && CurrentTarget.InLineOfSpellSight && !Me.HasAnyTempAura("Charge"), "[GapClose] Charge"))
            { await Coroutine.Sleep(800); return true; }

            return false;
        }

        public static async Task<bool> Pull()
        {
            if (CurrentTarget.IsValidCombatUnit())
            {
                await Movement.FaceCurrentTarget(CurrentTarget);
                await Movement.MoveToTarget(CurrentTarget);

                if (M.IS.AutoPull)
                {
                    if (await S.Cast(SB.HeroicThrow, C.Combat,
                        CurrentTarget.Distance <= 30 && CurrentTarget.Distance >= 8
                        && CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.InLineOfSight
                        && !Me.HasAnyTempAura("Charge")
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Heroic Throw"))
                    { return true; }

                    if (await S.Cast(SB.Charge, C.Combat,
                        CurrentTarget.Distance <= 25 && CurrentTarget.Distance >= 8
                        && CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.InLineOfSight
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Charge"))
                    { await Coroutine.Sleep(500); return true; }

                    if (await S.CastGround(SB.HeroicLeap, C.Combat,
                        CurrentTarget.Distance >= 15 && CurrentTarget.Distance <= 30
                        && CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.InLineOfSight
                        && !Me.HasAnyTempAura("Charge")
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Heroic Leap"))
                    { await Coroutine.Sleep(800); return true; }
                }

                if (await S.Cast(SB.HeroicThrow, C.Combat,
                    M.IS.FEnableHeroicThrow
                    && CurrentTarget.Distance <= 30 && CurrentTarget.Distance >= 8
                    && CurrentTarget.InLineOfSpellSight
                    && CurrentTarget.InLineOfSight
                    && !Me.HasAnyTempAura("Charge")                    
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Heroic Throw"))
                { return true; }

                if (await S.Cast(SB.Charge, C.Combat,
                    M.IS.FEnableCharge
                    && CurrentTarget.Distance >= 8 && CurrentTarget.Distance <= 25
                    && CurrentTarget.InLineOfSpellSight
                    && CurrentTarget.InLineOfSight   
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Charge"))
                { await Coroutine.Sleep(500); return true; }

                if (await S.CastGround(SB.HeroicLeap, C.Combat,
                    M.IS.FEnableHeroicLeap
                    && CurrentTarget.Distance <= 30 && CurrentTarget.Distance >= 15
                    && CurrentTarget.InLineOfSpellSight
                    && CurrentTarget.InLineOfSight
                    && !Me.HasAnyTempAura("Charge")                   
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Heroic Leap"))
                { await Coroutine.Sleep(800); return true; }
            }
            return false;
        }

        #endregion

        #region [Method] - Combat Buffing Behavior

        public static async Task<bool> CombatBuffing()
        {
            if (!Me.IsAlive || !Me.GotTarget || !CurrentTarget.IsAlive || Me.OnTaxi)
                return true;

            if (!Me.IsAutoAttacking && CurrentTarget.IsValidCombatUnit())
            {
                Lua.DoString("StartAttack()");
                return true;
            }
            await Coroutine.Yield();
            return false;
        }

        #endregion

        #region [Method] - Defensive Behavior

        public static async Task<bool> OnDefense()
        {
            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.HpBelow
                && CurrentTarget.Distance <= 10
                && Me.IsSafelyFacing(CurrentTarget)
                && M.IS.ShockwaveValue > 0
                && Me.HealthPercent < M.IS.ShockwaveValue,
                "[Defense] Shockwave"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.TargetCount
                && M.IS.ShockwaveValue > 0
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.ShockwaveValue,
                "[TC] Shockwave"))
            { return true; }

            if (await S.Gcd(SB.EnragedRegeneration, C.Combat,
                M.IS.EnableEnragedRegeneration
                && M.IS.EnragedRegenerationValue > 0
                && Me.HealthPercent < M.IS.EnragedRegenerationValue
                && U.ActiveEnemies(Me.Location, 10f).Any(),
                "[Defense] Enraged Regeneration"))
            { return true; }

            if (await S.Gcd(SB.CommandingShout, C.Combat,
                M.IS.FEnableCommandingShout
                && M.IS.FCommandingShoutValue > 0
                && Me.HealthPercent < M.IS.FCommandingShoutValue
                && U.ActiveEnemies(Me.Location, 10f).Any(),
                "[Defense] Commanding Shout (HP)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Combat,
                M.IS.EnableStormBolt
                && T.StormBolt
                && CurrentTarget.Distance <= 20
                && M.IS.StormBolt == SingleStunType.HpBelow
                && M.IS.StormBoltValue > 0
                && Me.HealthPercent < M.IS.StormBoltValue,
                "[Defense] Storm Bolt (HP)"))
            { return true; }

            if (await S.Gcd(SB.IntimidatingShout, C.Combat,
                M.IS.FEnableIntimidatingShout
                && M.IS.FIntimidatingShout == CrowdControlType.HpBelow
                && M.IS.FIntimidatingShoutValue > 0
                && Me.HealthPercent < M.IS.FIntimidatingShoutValue
                && U.ActiveEnemies(Me.Location, 8f).Any(),
                "[Defense] Intimidating Shout (HP)"))
            { await Coroutine.Sleep(2000); return true; }

            if (await S.Gcd(SB.IntimidatingShout, C.Combat,
                M.IS.FEnableIntimidatingShout
                && M.IS.FIntimidatingShout == CrowdControlType.TargetCount
                && M.IS.FIntimidatingShoutValue > 0
                && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.FIntimidatingShoutValue,
                "[Defense] Intimidating Shout (Target Count)"))
            { await Coroutine.Sleep(2000); return true; }

            return false;
        }

        #endregion

        #region [Method] - Opener Behaviors

        public static async Task<bool> Opener()
        {
            await S.Cast(SB.Charge, C.Combat, !CurrentTarget.IsWithinMeleeRangeOf(Me) && CurrentTarget.InLineOfSight && CurrentTarget.InLineOfSpellSight, "[OP] Charge");
            await S.Cast(SB.DragonRoar, C.Combat, T.DragonRoar && M.IS.EnableDragonRoar && M.IS.DragonRoarCd && CurrentTarget.IsWithinMeleeRangeOf(Me), "[OP] Dragon Roar");
            await S.Cast(SB.BattleCry, C.Combat, M.IS.FEnableBattleCry && M.IS.FBattleCryCd && CurrentTarget.IsWithinMeleeRangeOf(Me), "[OP] Battle Cry");
            await S.Cast(SB.Avatar, C.Combat, T.Avatar && M.IS.EnableAvatar && (M.IS.Avatar == ClassType.OnCooldown || (M.IS.Avatar == ClassType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite())) && CurrentTarget.IsWithinMeleeRangeOf(Me), "[OP] Avatar");
            return false;
        }

        #endregion

        #region [Method] - Cooldowns

        public static async Task<bool> Cooldowns()
        {
            if (await S.Gcd(SB.Shockwave, C.Combat,
                T.Shockwave
                && M.IS.EnableShockwave
                && M.IS.Shockwave == AoEStunType.OnCooldown
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10,
                "[CD] Shockwave"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Combat,
                M.IS.EnableStormBolt
                && T.StormBolt
                && CurrentTarget.Distance <= 20
                && M.IS.StormBolt == SingleStunType.OnCooldown,
                "[CD] Storm Bolt"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Single Target Behavior

        public static async Task<bool> SingleTarget()
        {
            if (await S.Gcd(SB.BerserkerRage, C.Combat,
                M.IS.EnableBerserkerRage
                && ((M.IS.BerserkerRage == ClassType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite())
                    || M.IS.BerserkerRage == ClassType.OnCooldown)
                && (Me.HasAuraWithEffect(WoWApplyAuraType.ModFear) || Me.HasAnyTempAura("Sap") || Me.HasAnyTempAura("Repentance") || Me.HasAnyTempAura("Gouge")),
                "[ST] Berserker Rage (Player/Boss/Elite)"))
            { return true; }

            if (await Opener()) { return true; }

            if (await S.Cast(SB.Rampage, C.Combat,
                ((!Me.HasAura(SB.auraEnrage) && CM.CurrentRage >= 85) || CM.CurrentRage > 99)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Rampage"))
            { return true; }

            if (await S.Cast(SB.FuriousSlash, C.Combat,
                M.IS.EnableFrenzy && T.Frenzy
                && ((Me.HasAura(SB.auraFrenzy) && U.AuraStackCount(Me, SB.auraFrenzy, true) < 3) || !Me.HasAura(SB.auraFrenzy))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Furious Slash (Frenzy Talent)"))
            { return true; }

            if (await S.Gcd(SB.FuryBladestorm, C.Combat,
                M.IS.FEnableBladestorm
                && T.Bladestorm
                && M.IS.FBladestormCd
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Bladestorm (CD)"))
            { return true; }

            if (await S.Cast(SB.Bloodthirst, C.Combat,
                !Me.HasAura(SB.auraEnrage)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Bloodthirst (Enrage Missing)"))
            { return true; }

            if (await S.Cast(SB.OdynsFury, C.Combat,
                Me.HasAura(SB.auraEnrage)
                && (Me.HasAura(SB.auraBattleCry) || Me.HasAura(SB.auraAvatar))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Odyn's Fury"))
            { return true; }

            if (await S.Cast(SB.RagingBlow, C.Combat,
                !T.InnerRage
                && Me.HasAura(SB.auraEnrage)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Raging Blow"))
            { return true; }

            if (await S.Cast(SB.RagingBlow, C.Combat,
                T.InnerRage
                && M.IS.EnableInnerRage
                && !_ragingBlowCast
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Raging Blow (Inner Rage)"))
            { _ragingBlowCast = true; return true; }

            await S.Cast(SB.Bloodthirst, C.Combat, CurrentTarget.IsWithinMeleeRangeOf(Me), "[ST] Bloodthirst (Enraged)");
            await S.Cast(SB.FuriousSlash, C.Combat, !T.Frenzy && CurrentTarget.IsWithinMeleeRangeOf(Me), "[ST] Furious Slash");

            _ragingBlowCast = false;
            return false;
        }

        #endregion

        #region [Method] - AOE Behavior

        public static async Task<bool> AoE()
        {
            if (await S.Gcd(SB.FuryBladestorm, C.Combat,
                M.IS.FEnableBladestorm
                && T.Bladestorm
                && (M.IS.FBladestormCd 
                    || (M.IS.FuryBurst == BurstType.TargetCount && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.FuryBurstValue))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Bladestorm"))
            { return true; }

            if (await S.Cast(SB.FuryWhirlwind, C.Combat,
                !Me.HasAura(SB.auraMeatCleaver)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Whirlwind (Meat Cleaver Activator)"))
            { return true; }

            if (await S.Cast(SB.Rampage, C.Combat,
                (!Me.HasAura(SB.auraEnrage) || (CM.CurrentRage > 99 && Me.HasAura(SB.auraMeatCleaver)))
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Rampage"))
            { return true; }

            if (await S.Cast(SB.Bloodthirst, C.Combat, !Me.HasAura(SB.auraEnrage) && Me.HasAura(SB.auraMeatCleaver) && CurrentTarget.IsWithinMeleeRangeOf(Me), "[AOE] Bloodthirst")) { return true; }
            if (await S.Cast(SB.RagingBlow, C.Combat, T.InnerRage && U.ActiveEnemies(Me.Location, 8f).Count() < 4 && CurrentTarget.IsWithinMeleeRangeOf(Me), "[AOE] Raging Blow")) { return true; }
            await S.Cast(SB.FuryWhirlwind, C.Combat, CurrentTarget.IsWithinMeleeRangeOf(Me), "[AOE] Whirlwind (Filler)");
            return false;
        }

        #endregion

        #region [Method] - Burst Behavior

        public static async Task<bool> Burst()
        {
            if (await S.Gcd(SB.BattleCry, C.Combat,
                M.IS.FEnableBattleCry
                && !M.IS.FBattleCryCd
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Battle Cry"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Combat,
                T.Shockwave
                && M.IS.EnableShockwave
                && M.IS.Shockwave == AoEStunType.OnBurst
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10,
                "[Burst] Shockwave"))
            { return true; }

            if (await S.Cast(SB.DragonRoar, C.Combat,
                M.IS.EnableDragonRoar
                && T.DragonRoar,
                "[Burst] Dragon Roar"))
            { return true; }

            if (await S.Cast(SB.OdynsFury, C.Combat,
                M.IS.EnableOdynsFury
                && !M.IS.OdynsFuryCd
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Odyn's Fury"))
            { return true; }

            if (await S.Gcd(SB.Avatar, C.Combat,
                M.IS.EnableAvatar
                && T.Avatar
                && M.IS.Avatar == ClassType.OnBurst
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Avatar"))
            { return true; }

            if (await S.Cast(SB.Bloodbath, C.Combat,
                M.IS.EnableBloodbath
                && T.Bloodbath
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Bloodbath"))
            { return true; }

            if (await S.Gcd(SB.FuryBladestorm, C.Combat,
                M.IS.FEnableBladestorm
                && T.Bladestorm
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Burst] Bladestorm"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Interrupt Behavior

        public static async Task<bool> Interrupt()
        {
            if (!M.IS.AllowFuryInterrupts)
                return false;

            #region Interrupt Heal Cast

            if (await S.Cast(SB.Pummel, C.Interrupt,
                M.IS.PummelHeal
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Pummel (Heal)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Interrupt,
                M.IS.StormBoltHeal
                && T.StormBolt
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.Distance <= 20
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Storm Bolt (Heal)"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Interrupt,
                M.IS.ShockwaveHeal
                && T.Shockwave
                && CurrentTarget.IsValidCombatUnit()
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Shockwave (Heal)"))
            { return true; }

            if (await S.Gcd(SB.IntimidatingShout, C.Interrupt,
                M.IS.FuryIntimidatingShoutHeal
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Intimidating Shout (Heal)"))
            { return true; }

            #endregion

            #region Interrupt Any Cast

            if (await S.Cast(SB.Pummel, C.Interrupt,
                M.IS.PummelAny
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Pummel (Any)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Interrupt,
                M.IS.StormBoltAny
                && T.StormBolt
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.Distance <= 20
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Storm Bolt (Any)"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Interrupt,
                M.IS.ShockwaveAny
                && T.Shockwave
                && CurrentTarget.IsValidCombatUnit()
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Shockwave (Any)"))
            { return true; }

            if (await S.Gcd(SB.IntimidatingShout, C.Interrupt,
                M.IS.FuryIntimidatingShoutAny
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Intimidating Shout (Any)"))
            { return true; }

            #endregion

            return false;
        }

        #endregion

        #region [Method] - Casting Targets

        private static WoWUnit GetTargetsHealing(double range)
        {
            var units = U.ActiveEnemies(Me.Location, range);
            var target =
                units?.Where(u => u.IsCastingHealingSpell && u.CanInterruptCurrentSpellCast)
                    .OrderBy(d => d.Distance)
                    .FirstOrDefault();

            return target;
        }

        private static WoWUnit GetTargetCasting(double range)
        {
            var units = U.ActiveEnemies(Me.Location, range);

            var target =
                units?.Where(u => (u.IsCasting || u.IsCastingHealingSpell) && u.CanInterruptCurrentSpellCast)
                    .OrderBy(d => d.Distance)
                    .FirstOrDefault();
            return target;
        }

        #endregion

        #region [Method] - Conditions

        private static bool BurstCondition()
        {
            return (M.IS.FuryBurst == BurstType.HpBelow && M.IS.FuryBurstValue > 0 && Me.HealthPercent < M.IS.FuryBurstValue)
                    || (M.IS.FuryBurst == BurstType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite())
                    || (M.IS.FuryBurst == BurstType.TargetCount && M.IS.FuryBurstValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.FuryBurstValue)
                    || (M.IS.FuryBurst == BurstType.TargetHpBelow && M.IS.FuryBurstValue > 0 && CurrentTarget.HealthPercent < M.IS.FuryBurstValue)
                    || M.IS.FuryBurst == BurstType.OnCooldown
                    || (M.IS.FuryBurst == BurstType.BurstKeys && HotkeyManagement.BurstOn);
        }

        #endregion
    }
}
