﻿using System.Linq;
using System.Threading.Tasks;
using Buddy.Coroutines;
using IronBreaker.Core;
using IronBreaker.Core.Helpers;
using IronBreaker.Core.Utilities;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

#region Class Redundancy
using C = IronBreaker.Core.Utilities.Coloring;
using CM = IronBreaker.Core.Helpers.Common;
using M = IronBreaker.Main;
using SB = IronBreaker.Core.Helpers.SpellBook;
using S = IronBreaker.Core.Spell;
using U = IronBreaker.Core.Unit;
using T = IronBreaker.Core.Managers.TalentManagement;
#endregion

namespace IronBreaker.Rotations
{
    internal class Protection
    {
        #region Declarations
        private static LocalPlayer Me => StyxWoW.Me;
        private static WoWUnit CurrentTarget => StyxWoW.Me.CurrentTarget;
        #endregion

        #region [Method] - Rotation Selector

        public static async Task<bool> RotationSelector()
        {
            if (!Me.IsAlive || Me.OnTaxi || Me.IsCasting || Me.IsChanneling)
                return true;

            await Pull();
            await Interrupt();
            await Movement.FaceCurrentTarget(CurrentTarget);
            await Movement.MoveToTarget(CurrentTarget);

            if (await S.Gcd(SB.Taunt, C.Tank,
                M.IS.AutoTaunt
                && !S.OnCooldown(SB.Taunt)
                && M.CurrentContext == WoWContext.Instances
                && CurrentTarget.IsValidCombatUnit()
                && !CurrentTarget.IsTargetingMeOrPet,
                "[Tank] Taunting"))
            {
                if (CurrentTarget.IsTargetingMeOrPet)
                    Me.TargetLastTarget();
                return true;
            }

            if (CurrentTarget.IsValidCombatUnit())
            {
                if (await ActiveMitigation()) { return true; }
                if (await Cooldowns()) { return true; }

                if ((M.IS.AutoAoE && M.IS.AoEValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.AoEValue)
                    || (!M.IS.AutoAoE && U.ActiveEnemies(Me.Location, 8f).Count() >= 3))
                {
                    if (await AoE())
                        return true;
                }

                return await SingleTarget();
            }
            return false;
        }

        #endregion

        #region [Method] - PreCombat Buffing Behavior

        public static async Task<bool> PreCombatBuffing()
        {
            if (Me.Combat || !Me.IsAlive || Me.OnTaxi)
                return true;

            await Coroutine.Yield();
            return false;
        }

        #endregion

        #region [Method] Pulling / Gap Closer Behavior

        public static async Task<bool> Pull()
        {
            if (CurrentTarget.IsValidCombatUnit())
            {
                await Movement.FaceCurrentTarget(CurrentTarget);
                await Movement.MoveToTarget(CurrentTarget);

                if (M.IS.AutoPull)
                {
                    if (await S.Cast(SB.HeroicThrow, C.Combat,
                        CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.Distance <= 30
                        && CurrentTarget.Distance >= 8
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Heroic Throw"))
                    { return true; }

                    if (await S.Cast(SB.Intercept, C.Combat,
                        CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.Distance >= 8
                        && CurrentTarget.Distance <= 25
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Intercept"))
                    { return true; }

                    if (await S.CastGround(SB.HeroicLeap, C.Combat,
                        CurrentTarget.InLineOfSpellSight
                        && CurrentTarget.InLineOfSight
                        && CurrentTarget.Distance >= 15
                        && CurrentTarget.Distance <= 30
                        && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                        "[AutoPull] Heroic Leap"))
                    { await Coroutine.Sleep(800); return true; }
                }

                if (await S.Cast(SB.HeroicThrow, C.Combat,
                    CurrentTarget.InLineOfSpellSight
                    && M.IS.PEnableHeroicThrow
                    && CurrentTarget.Distance <= 30
                    && CurrentTarget.Distance >= 8
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Heroic Throw"))
                { return true; }

                if (await S.Cast(SB.Intercept, C.Combat,
                    CurrentTarget.InLineOfSpellSight
                    && M.IS.EnableIntercept
                    && CurrentTarget.Distance >= 8
                    && CurrentTarget.Distance <= 25
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[AutoPull] Intercept"))
                { return true; }

                if (await S.CastGround(SB.HeroicLeap, C.Combat,
                    M.IS.PEnableHeroicLeap
                    && CurrentTarget.InLineOfSpellSight
                    && CurrentTarget.InLineOfSight
                    && CurrentTarget.Distance >= 15
                    && CurrentTarget.Distance <= 30
                    && !CurrentTarget.IsWithinMeleeRangeOf(Me),
                    "[Pull] Heroic Leap"))
                { await Coroutine.Sleep(800); return true; }
            }
            return false;
        }

        #endregion

        #region [Method] - Combat Buffing Behavior

        public static async Task<bool> CombatBuffing()
        {
            if (!Me.IsAlive || !Me.GotTarget || !CurrentTarget.IsAlive || Me.OnTaxi)
                return true;

            if (!Me.IsAutoAttacking && CurrentTarget.IsValidCombatUnit())
            {
                Lua.DoString("StartAttack()");
                return true;
            }
            await Coroutine.Yield();
            return false;
        }

        #endregion

        #region [Method] - Active Mitigation Behavior

        public static async Task<bool> ActiveMitigation()
        {
            #region Defensive Cooldowns
            if (await S.Gcd(SB.LastStand, C.Combat,
                M.IS.EnableLastResort
                && M.IS.LastResortValue > 0
                && Me.HealthPercent < M.IS.LastResortValue
                && U.ActiveEnemies(Me.Location, 8f).Any(),
                "[Buff] Last Stand"))
            { return true; }

            if (await S.Gcd(SB.ShieldWall, C.Combat,
                M.IS.EnableShieldWall
                && M.IS.ShieldWallValue > 0
                && Me.HealthPercent < M.IS.ShieldWallValue
                && U.ActiveEnemies(Me.Location, 8f).Any(),
                "[Buff] Shield Wall"))
            { return true; }

            if (await S.Gcd(SB.BattleCry, C.Combat,
                M.IS.PEnableBattleCry
                && ((M.IS.PBattleCry == BuffType.HpBelow && M.IS.PBattleCryValue > 0 && Me.HealthPercent < M.IS.PBattleCryValue)
                    || (M.IS.PBattleCry == BuffType.TargetCount && M.IS.PBattleCryValue > 0 && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.PBattleCryValue)
                    || (M.IS.PBattleCry == BuffType.PlayerBossElite && CurrentTarget.IsPlayerBossOrElite())
                    || M.IS.PBattleCry == BuffType.OnCooldown)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Battle Cry"))
            { return true; }

            if (await S.Cast(SB.NeltharionsFury, C.Combat,
                M.IS.EnableNeltharionsRage
                && M.IS.NeltharionsRage == BuffType.HpBelow
                && M.IS.NeltharionsRageValue > 0
                && Me.HealthPercent < M.IS.NeltharionsRageValue
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Mitigation] Neltharion's Fury"))
            { return true; }

            #endregion

            #region Ignore Pain

            if (await S.Gcd(SB.IgnorePain, C.Combat,
                M.IS.EnableIgnorePain
                && T.Vengeance
                && !Me.HasAura("Ignore Pain")
                && !Me.HasAura("Vengeance: Ignore Pain")
                && CM.CurrentRage >= 20
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Initial Ignore Pain (Talented)"))
            {
                return true;
            }

            if (await S.Gcd(SB.IgnorePain, C.Combat,
                M.IS.EnableIgnorePain
                && T.Vengeance
                && !Me.HasAura("Ignore Pain")
                && Me.HasAura("Vengeance: Ignore Pain")
                && CM.CurrentRage >= 30
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Ignore Pain (Talented)"))
            {
                return true;
            }

            if (await S.Gcd(SB.IgnorePain, C.Combat,
                M.IS.EnableIgnorePain
                && !T.Vengeance
                && !Me.HasAura("Ignore Pain")
                && CM.CurrentRage >= 40
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Ignore Pain (NonTalented)"))
            {
                return true;
            }

            #endregion

            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.TargetCount
                && M.IS.ShockwaveValue > 0
                && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.ShockwaveValue
                && Me.IsSafelyFacing(CurrentTarget),
                "[Mitigation] Shockwave (Target Count)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Combat,
                M.IS.EnableStormBolt
                && T.StormBolt
                && M.IS.StormBolt == SingleStunType.HpBelow
                && M.IS.StormBoltValue > 0
                && Me.HealthPercent < M.IS.StormBoltValue
                && CurrentTarget.Distance <= 20,
                "[Mitigation] Storm Bolt (HP)"))
            { return true; }

            //if (await S.Gcd(SB.Avatar, C.Combat,
            //    M.IS.EnableAvatar
            //    && T.Avatar
            //    && M.IS.Avatar == AvatarType.HpBelow
            //    && M.IS.AvatarValue > 0
            //    && Me.HealthPercent < M.IS.AvatarValue
            //    && CurrentTarget.IsWithinMeleeRangeOf(Me),
            //    "[Mitigation] Avatar (HP)"))
            //{ return true; }

            if (await S.Cast(SB.ImpendingVictory, C.Combat,
                M.IS.EnableImpendingVictory
                && T.ImpendingVictory
                && M.IS.ImpendingVictory == ImpendingVictoryType.HpBelow
                && M.IS.ImpendingVictoryValue > 0
                && Me.HealthPercent < M.IS.ImpendingVictoryValue
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Mitigation] Impending Victory"))
            { return true; }

            if (await S.Cast(SB.ProtRavager, C.Combat,
                M.IS.PEnableRavager
                && T.Ravager
                && M.IS.PRavager == DefenseType.HpBelow
                && M.IS.PRavagerValue > 0
                && Me.HealthPercent < M.IS.PRavagerValue
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Mitigation] Ravager"))
            { return true; }

            #region Demoralizing Shout

            if (await S.Gcd(SB.DemoralizingShout, C.Combat,
                M.IS.EnableDemoralizingShout
                && M.IS.DemoralizingShoutValue > 0
                && Me.HealthPercent < M.IS.DemoralizingShoutValue
                && U.ActiveEnemies(Me.Location, 8f).Any(),
                "[Mitigation] Demoralizing Shout"))
            { return true; }

            if (await S.Gcd(SB.DemoralizingShout, C.Combat,
                M.IS.EnableDemoralizingShout
                && M.IS.DemoralizingShoutValue == 0
                && Me.HealthPercent < 60
                && U.ActiveEnemies(Me.Location, 8f).Any(),
                "[Mitigation] Demoralizing Shout (60% HP)"))
            { return true; }

            #endregion

            #region Shield Block

            if (await S.Gcd(SB.ShieldBlock, C.Combat,
                M.IS.EnableShieldBlock
                && M.IS.ShieldBlockValue > 0
                && Me.HealthPercent < M.IS.ShieldBlockValue + 10
                && CM.CurrentRage >= 10
                && S.GetCharges(SB.ShieldBlock) >= 2
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Mitigation] Shield Block"))
            { return true; }

            if (await S.Gcd(SB.ShieldBlock, C.Combat,
                M.IS.EnableShieldBlock
                && M.IS.ShieldBlockValue > 0
                && Me.HealthPercent < M.IS.ShieldBlockValue
                && S.GetCharges(SB.ShieldBlock) >= 1
                && CM.CurrentRage >= 10
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Mitigation] Shield Block"))
            { return true; }

            if (await S.Gcd(SB.ShieldBlock, C.Combat,
                M.IS.EnableShieldBlock
                && M.IS.ShieldBlockValue == 0
                && Me.HealthPercent < 80
                && CM.CurrentRage >= 10
                && S.GetCharges(SB.ShieldBlock) >= 2
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Migitation] Shield Block"))
            { return true; }

            if (await S.Gcd(SB.ShieldBlock, C.Combat,
                M.IS.EnableShieldBlock
                && M.IS.ShieldBlockValue == 0
                && Me.HealthPercent < 40
                && CM.CurrentRage >= 10
                && S.GetCharges(SB.ShieldBlock) >= 1
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Mitigation] Shield Block"))
            { return true; }

            #endregion

            if (await S.Gcd(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.HpBelow
                && M.IS.ShockwaveValue > 0
                && Me.HealthPercent < M.IS.ShockwaveValue
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10,
                "[Mitigation] Shockwave (HP)"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Cooldowns

        public static async Task<bool> Cooldowns()
        {
            if (await S.Gcd(SB.BerserkerRage, C.Combat,
                M.IS.EnableBerserkerRage
                && M.IS.BerserkerRage == ClassType.OnCooldown
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (Me.HasAuraWithEffect(WoWApplyAuraType.ModFear) || Me.HasAnyTempAura("Sap") || Me.HasAnyTempAura("Repentance") || Me.HasAnyTempAura("Gouge")),
                "[Buff] Berserker Rage"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Combat,
                M.IS.EnableStormBolt
                && T.StormBolt
                && M.IS.StormBolt == SingleStunType.OnCooldown
                && CurrentTarget.Distance <= 20,
                "[CD] Storm Bolt"))
            { return true; }

            if (await S.Gcd(SB.Avatar, C.Combat,
                M.IS.EnableAvatar
                && T.Avatar
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[CD] Avatar"))
            { return true; }

            if (await S.Cast(SB.ProtRavager, C.Combat,
                M.IS.PEnableRavager
                && T.Ravager
                && M.IS.PRavager == DefenseType.OnCooldown
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[CD] Ravager"))
            { return true; }

            if (await S.Cast(SB.NeltharionsFury, C.Combat,
                M.IS.EnableNeltharionsRage
                && M.IS.NeltharionsRage == BuffType.OnCooldown
                && U.ActiveEnemies(Me.Location, 8f).Any()
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[CD] Neltharion's Fury"))
            { return true; }

            if (await S.Cast(SB.Shockwave, C.Combat,
                M.IS.EnableShockwave
                && T.Shockwave
                && M.IS.Shockwave == AoEStunType.OnCooldown
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10,
                "[CD] Shockwave"))
            { return true; }

            if (await S.Cast(SB.ImpendingVictory, C.Combat,
                M.IS.EnableImpendingVictory
                && T.ImpendingVictory
                && M.IS.ImpendingVictory == ImpendingVictoryType.OnCooldown
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[CD] Impending Victory"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Single Target Behavior

        public static async Task<bool> SingleTarget()
        {
            if (await S.Cast(SB.VictoryRush, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && Me.HasAura("Victorious"),
                "Victory Rush Proc"))
            { return true; }

            #region Bigger / CD Abilities

            if (await S.Gcd(SB.Avatar, C.Combat,
                M.IS.EnableAvatar
                && T.Avatar
                && M.IS.Avatar == ClassType.PlayerBossElite
                && CurrentTarget.IsPlayerBossOrElite()
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Avatar (Player/Boss/Elite)"))
            { return true; }

            if (await S.Cast(SB.ImpendingVictory, C.Combat,
                M.IS.EnableImpendingVictory
                && T.ImpendingVictory
                && M.IS.ImpendingVictory == ImpendingVictoryType.PlayerBossElite
                && (CurrentTarget.IsPlayer || CurrentTarget.IsBoss || CurrentTarget.Elite)
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Impending Victory"))
            { return true; }

            if (await S.Cast(SB.NeltharionsFury, C.Combat,
                M.IS.EnableNeltharionsRage
                && M.IS.NeltharionsRage == BuffType.PlayerBossElite
                && CurrentTarget.IsPlayerBossOrElite()
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Neltharion's Fury"))
            { return true; }

            #endregion

            if (await S.Cast(SB.FocusedRage, C.Combat,
                T.Ultimatum
                && Me.HasAura(SB.auraUltimatum),
                "[Buff] Focused Rage (Ultimatum Proc)"))
            { return true; }

            if (await S.Cast(SB.ShieldSlam, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Shield Slam"))
            { return true; }

            if (await S.Cast(SB.FocusedRage, C.Combat,
                T.Vengeance
                && Me.HasAura("Vengeance: Focused Rage")
                && CM.CurrentRage >= 20
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Focused Rage (Talented)"))
            { return true; }

            if (await S.Cast(SB.FocusedRage, C.Combat,
                !T.Vengeance
                && CM.CurrentRage >= 30
                && Me.HealthPercent > 65
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Focused Rage (NonTalented)"))
            { return true; }

            if (await S.Cast(SB.ShieldSlam, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Shield Slam"))
            { return true; }

            if (await S.Cast(SB.Revenge, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Revenge"))
            { return true; }

            if (await S.Cast(SB.Devastate, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && S.OnCooldown(SB.ShieldSlam),
                "[ST] Devastate"))
            { return true; }

            if (await S.Cast(SB.Revenge, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && S.OnCooldown(SB.ShieldSlam)
                && S.CooldownTimeLeft(SB.ShieldSlam) <= 1500,
                "[ST] Revenge"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - AOE Behavior

        public static async Task<bool> AoE()
        {
            if (await S.Cast(SB.VictoryRush, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && Me.HasAura("Victorious"),
                "Victory Rush Proc"))
            { return true; }

            if (await S.Gcd(SB.ThunderClap, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Thunder Clap"))
            { return true; }

            #region Bigger / CD Abilities

            if (await S.Cast(SB.NeltharionsFury, C.Combat,
                M.IS.EnableNeltharionsRage
                && M.IS.NeltharionsRage == BuffType.TargetCount
                && M.IS.NeltharionsRageValue > 0
                && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.NeltharionsRageValue
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[AOE] Neltharion's Fury"))
            { return true; }

            if (await S.Cast(SB.ProtRavager, C.Combat,
                M.IS.PEnableRavager
                && T.Ravager
                && M.IS.PRavager == DefenseType.TargetCount
                && M.IS.PRavagerValue > 0
                && U.ActiveEnemies(Me.Location, 8f).Count() >= M.IS.PRavagerValue,
                "[AOE] Ravager"))
            { return true; }

            #endregion

            if (await S.Cast(SB.FocusedRage, C.Combat,
                T.Ultimatum
                && Me.HasAura(SB.auraUltimatum),
                "[Buff] Focused Rage (Ultimatum Proc)"))
            { return true; }

            if (await S.Cast(SB.ShieldSlam, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Shield Slam"))
            { return true; }

            if (await S.Cast(SB.FocusedRage, C.Combat,
                T.Vengeance
                && Me.HasAura("Vengeance: Focused Rage")
                && CM.CurrentRage >= 20
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Focused Rage (Talented)"))
            { return true; }

            if (await S.Cast(SB.FocusedRage, C.Combat,
                !T.Vengeance
                && CM.CurrentRage >= 30
                && Me.HealthPercent > 65
                && CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[Buff] Focused Rage (NonTalented)"))
            { return true; }

            if (await S.Cast(SB.ShieldSlam, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Shield Slam"))
            { return true; }

            if (await S.Cast(SB.Revenge, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me),
                "[ST] Revenge"))
            { return true; }

            if (await S.Cast(SB.Devastate, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && S.OnCooldown(SB.ShieldSlam),
                "[ST] Devastate"))
            { return true; }

            if (await S.Cast(SB.Revenge, C.Combat,
                CurrentTarget.IsWithinMeleeRangeOf(Me)
                && S.OnCooldown(SB.ShieldSlam)
                && S.CooldownTimeLeft(SB.ShieldSlam) <= 1500,
                "[ST] Revenge"))
            { return true; }

            return false;
        }

        #endregion

        #region [Method] - Interrupt Behavior

        public static async Task<bool> Interrupt()
        {
            if (!M.IS.AllowProtInterrupts)
                return false;

            #region Interrupt Heal Cast

            if (await S.Cast(SB.Pummel, C.Interrupt,
                M.IS.PummelHeal
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Pummel (Heal)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Interrupt,
                M.IS.StormBoltHeal
                && T.StormBolt
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.Distance <= 20
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Storm Bolt (Heal)"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Interrupt,
                M.IS.ShockwaveHeal
                && T.Shockwave
                && CurrentTarget.IsValidCombatUnit()
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && CurrentTarget.IsCastingHealingSpell
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Shockwave (Heal)"))
            { return true; }

            #endregion

            #region Interrupt Any Cast

            if (await S.Cast(SB.Pummel, C.Interrupt,
                M.IS.PummelAny
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.IsWithinMeleeRangeOf(Me)
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Pummel (Any)"))
            { return true; }

            if (await S.Cast(SB.StormBolt, C.Interrupt,
                M.IS.StormBoltAny
                && T.StormBolt
                && CurrentTarget.IsValidCombatUnit()
                && CurrentTarget.Distance <= 20
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Storm Bolt (Any)"))
            { return true; }

            if (await S.Gcd(SB.Shockwave, C.Interrupt,
                M.IS.ShockwaveAny
                && T.Shockwave
                && CurrentTarget.IsValidCombatUnit()
                && Me.IsSafelyFacing(CurrentTarget)
                && CurrentTarget.Distance <= 10
                && (CurrentTarget.IsCasting || CurrentTarget.IsCastingHealingSpell)
                && CurrentTarget.CanInterruptCurrentSpellCast,
                "[Interrupt] Shockwave (Any)"))
            { return true; }

            #endregion

            return false;
        }

        #endregion
    }
}
