﻿using System;
using System.IO;
using System.ComponentModel;
using System.Diagnostics;
using IronBreaker.Core.Utilities;
using Styx.Helpers;

#region Class Redundancy

using DefaultValue = Styx.Helpers.DefaultValueAttribute;

#endregion

namespace IronBreaker.Settings
{
    public class IronBreakerSettings : Styx.Helpers.Settings, INotifyPropertyChanged
    {
        [Setting, DefaultValue("0")]
        public string CurrentRevision { get; set; }

        #region Declarations
        private static IronBreakerSettings _instance;
        public static IronBreakerSettings Instance => _instance ?? (_instance = new IronBreakerSettings());
        public new static string SettingsPath => string.Format($"{CharacterSettingsDirectory}\\{Main.CrName}");
        public new event PropertyChangedEventHandler PropertyChanged;
        #endregion

        #region ComboBox Types
        public AoEStunType Shockwave { get; set; }
        public SingleStunType StormBolt { get; set; }
        public InterruptType Pummel { get; set; }
        public ClassType Avatar { get; set; }
        public ClassType BerserkerRage { get; set; }
        public DefenseType DefensiveStance { get; set; }
        public DefenseType PRavager { get; set; }
        public BurstType ArmsBurst { get; set; }
        public ArmsType ABladestorm { get; set; }
        public ArmsType Ravager { get; set; }
        public BurstType FuryBurst { get; set; }
        public CrowdControlType AIntimidatingShout { get; set; }             
        public CrowdControlType FIntimidatingShout { get; set; }               
        public BuffType PBattleCry { get; set; }
        public BuffType NeltharionsRage { get; set; }
        public BuffType ShieldReflection { get; set; }
        public ImpendingVictoryType ImpendingVictory { get; set; }
        #endregion

        public IronBreakerSettings() : base(Path.Combine(CharacterSettingsDirectory, Main.CrName + "Routine.xml"))
        {           
        }

        ~IronBreakerSettings()
        {
            Save();
        }

        #region [Binding] - Data Binding

        #region ReadOnly Values

        private bool _changelog;
        private bool _general;
        private bool _arms;
        private bool _fury;
        private bool _prot;
        private bool _hotkeys;
        
        #region CheckBox's
        private bool _autoMovement;
        private bool _autoTarget;
        private bool _autoFace;
        private bool _autoTaunt;
        private bool _autoPull;
        private bool _autoAoe;
        private bool _shockwave;
        private bool _stormBolt;
        private bool _avatar;
        private bool _berserkerRage;
        private bool _forceCombat;
        private bool _ignorePvp;
        private bool _pummel;
        private bool _resting;
        private bool _combatLog;
        private bool _debugLog;
        private bool _stoneHeart;
        private bool _aBattleCry;
        private bool _aBladestorm;
        private bool _ravager;
        private bool _aFocusedRage;
        private bool _warbreaker;
        private bool _aCharge;
        private bool _aHeroicLeap;
        private bool _aHeroicThrow;
        private bool _dieByTheSword;
        private bool _aCommandingShout;
        private bool _aIntimidatingShout;
        private bool _defensiveStance;
        private bool _fBattleCry;
        private bool _fBattleCryCd;
        private bool _odynsFury;
        private bool _odynsFuryCd;
        private bool _enragedRegeneration;
        private bool _fCommandingShout;
        private bool _fIntimidatingShout;
        private bool _fHeroicLeap;
        private bool _fCharge;
        private bool _fHeroicThrow;
        private bool _demoralizingShout;
        private bool _ignorePain;
        private bool _shieldWall;
        private bool _lastResort;
        private bool _shieldBlock;
        private bool _victoryRush;
        private bool _neltharionsRage;
        private bool _pBattleCry;
        private bool _pHeroicThrow;
        private bool _pHeroicLeap;
        private bool _intercept;      
        private bool _fBladestorm;
        private bool _dragonRoar;
        private bool _bloodBath;
        private bool _impendingVictory;
        private bool _pRavager;
        private bool _shieldReflection;
        private bool _dragonRoarCd;
        private bool _fBladestormCd;
        private bool _fBloodbathCd;
        private bool _innerRage;
        private bool _frenzy;
        #endregion

        #region Radio Buttons

        private bool _aBattleCryCd;
        private bool _aBattleCryAuto;
        private bool _warbreakerAuto;
        private bool _warbreakerUnitCount;

        #endregion

        #region TextBox's
        private int _aoeValue;
        private int _shockwaveValue;
        private int _stormBoltValue;
        private int _restingValue;
        private int _aBurstValue;
        private int _aBladestormValue;
        private int _ravagerValue;
        private int _warbreakerValue;
        private int _dieByTheSwordValue;
        private int _aCommandingShoutValue;     
        private int _aIntimidatingShoutValue;
        private int _defensiveStanceValue;
        private int _fBurstValue;
        private int _enragedRegenerationValue;
        private int _fCommandingShoutValue;
        private int _fIntimidatingShoutValue;
        private int _demoralizingShoutValue;
        private int _shieldWallValue;
        private int _lastResortValue;
        private int _shieldReflectionValue;
        private int _shieldBlockValue;
        private int _neltharionsRageValue;
        private int _pBattleCryValue;      
        private int _impendingVictoryValue;
        private int _pRavagerValue;
        #endregion

        #region ComboBox's
        private int _shockwaveCondition;
        private int _stormBoltCondition;
        private int _avatarCondition;
        private int _berserkerRageCondition;
        private int _pummelCondition;                
        private int _aBurstCondition;
        private int _aBladestormCondition;
        private int _ravagerCondition;     
        private int _aIntimidatingShoutCondition;
        private int _defensiveStanceCondition;
        private int _fBurstCondition;
        private int _fIntimidatingShoutCondition;
        private int _pBattleCryCondition;        
        private int _impendingVictoryCondition;
        private int _pRavagerCondition;
        private int _neltharionsRageCondition;
        private int _shieldReflectionCondition;
        #region Hotkeys
        private int _hkAutoModifier;
        private int _autoModifierKey;
        private int _hotkeyAbilitiesModifier;
        private int _abilitiesModifierKey;
        private int _hotkeyFacing;
        private int _facingKey;
        private int _hotkeyTarget;
        private int _targetKey;
        private int _hotkeyMovement;
        private int _movementKey;
        private int _hotkeyPull;
        private int _pullKey;
        private int _hotkeyAoE;
        private int _aoeKey;
        private int _hotkeyBurst;
        private int _burstKey;
        #endregion
        #endregion

        #endregion

        #region Tab Control Settings

        [DefaultValue(true)]
        public bool ChangelogSelected
        {
            get { return _changelog; }
            set { if (value.Equals(_changelog)) return; _changelog = value; Notify(); }
        }

        [DefaultValue(false)]
        public bool GeneralSelected
        {
            get { return _general; }
            set { if (value.Equals(_general)) return; _general = value; Notify(); }
        }

        [DefaultValue(false)]
        public bool ArmsSelected
        {
            get { return _arms; }
            set { if (value.Equals(_arms)) return; _arms = value; Notify(); }
        }

        [DefaultValue(false)]
        public bool FurySelected
        {
            get { return _fury; }
            set { if (value.Equals(_fury)) return; _fury = value; Notify(); }
        }

        [DefaultValue(false)]
        public bool ProtSelected
        {
            get { return _prot; }
            set { if (value.Equals(_prot)) return; _prot = value; Notify(); }
        }

        [DefaultValue(false)]
        public bool HotkeysSelected
        {
            get { return _hotkeys; }
            set { if (value.Equals(_hotkeys)) return; _hotkeys = value; Notify(); }
        }

        #endregion

        #region [Settings] - CheckBox's

        #region [TabItem] - General

        [Setting, DefaultValue(true)]
        public bool AutoMove
        {
            get { return _autoMovement; }
            set { if (value.Equals(_autoMovement)) return; _autoMovement = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AutoFace
        {
            get { return _autoFace; }
            set { if (value.Equals(_autoFace)) return; _autoFace = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AutoTarget
        {
            get { return _autoTarget; }
            set { if (value.Equals(_autoTarget)) return; _autoTarget = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AutoTaunt
        {
            get { return _autoTaunt; }
            set { if (value.Equals(_autoTaunt)) return; _autoTaunt = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AutoPull
        {
            get { return _autoPull; }
            set { if (value.Equals(_autoPull)) return; _autoPull = value; Notify(); }
        }

        //[Setting, DefaultValue(true)]
        //public bool MultiPull
        //{
        //    get { return _multiPull; }
        //    set { if (value.Equals(_multiPull)) return; _multiPull = value; Notify(); }
        //}

        [Setting, DefaultValue(true)]
        public bool AutoAoE
        {
            get { return _autoAoe; }
            set { if (value.Equals(_autoAoe)) return; _autoAoe = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableShockwave
        {
            get { return _shockwave; }
            set { if (value.Equals(_shockwave)) return; _shockwave = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableStormBolt
        {
            get { return _stormBolt; }
            set { if (value.Equals(_stormBolt)) return; _stormBolt = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableAvatar
        {
            get { return _avatar; }
            set { if (value.Equals(_avatar)) return; _avatar = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableBerserkerRage
        {
            get { return _berserkerRage; }
            set { if (value.Equals(_berserkerRage)) return; _berserkerRage = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool ForceCombat
        {
            get { return _forceCombat; }
            set { if (value.Equals(_forceCombat)) return; _forceCombat = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool IgnorePvP
        {
            get { return _ignorePvp; }
            set { if (value.Equals(_ignorePvp)) return; _ignorePvp = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnablePummel
        {
            get { return _pummel; }
            set { if (value.Equals(_pummel)) return; _pummel = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableResting
        {
            get { return _resting; }
            set { if (value.Equals(_resting)) return; _resting = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableCombatLog
        {
            get { return _combatLog; }
            set { if (value.Equals(_combatLog)) return; _combatLog = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableDebugLog
        {
            get { return _debugLog; }
            set { if (value.Equals(_debugLog)) return; _debugLog = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableStoneHeart
        {
            get { return _stoneHeart; }
            set { if (value.Equals(_stoneHeart)) return; _stoneHeart = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Arms

        [Setting, DefaultValue(true)]
        public bool AEnableBattleCry
        {
            get { return _aBattleCry; }
            set { if (value.Equals(_aBattleCry)) return; _aBattleCry = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool ABattleCryCd
        {
            get { return _aBattleCryCd; }
            set { _aBattleCryCd = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool ABattleCryAuto
        {
            get { return _aBattleCryAuto; }
            set { _aBattleCryAuto = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AEnableBladeStorm
        {
            get { return _aBladestorm; }
            set { if (value.Equals(_aBladestorm)) return; _aBladestorm = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableRavager
        {
            get { return _ravager; }
            set { if (value.Equals(_ravager)) return; _ravager = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableWarbreaker
        {
            get { return _warbreaker; }
            set { if (value.Equals(_warbreaker)) return; _warbreaker = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool WarbreakerAuto
        {
            get { return _warbreakerAuto; }
            set { _warbreakerAuto = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool WarbreakerUnitCount
        {
            get { return _warbreakerUnitCount; }
            set { _warbreakerUnitCount = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool AEnableFocusedRage
        {
            get { return _aFocusedRage; }
            set { if (value.Equals(_aFocusedRage)) return; _aFocusedRage = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AEnableCharge
        {
            get { return _aCharge; }
            set { if (value.Equals(_aCharge)) return; _aCharge = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AEnableHeroicLeap
        {
            get { return _aHeroicLeap; }
            set { if (value.Equals(_aHeroicLeap)) return; _aHeroicLeap = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AEnableHeroicThrow
        {
            get { return _aHeroicThrow; }
            set { if (value.Equals(_aHeroicThrow)) return; _aHeroicThrow = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableDieByTheSword
        {
            get { return _dieByTheSword; }
            set { if (value.Equals(_dieByTheSword)) return; _dieByTheSword = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool AEnableCommandingShout
        {
            get { return _aCommandingShout; }
            set { if (value.Equals(_aCommandingShout)) return; _aCommandingShout = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool AEnableIntimidatingShout
        {
            get { return _aIntimidatingShout; }
            set { if (value.Equals(_aIntimidatingShout)) return; _aIntimidatingShout = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableDefensiveStance
        {
            get { return _defensiveStance; }
            set { if (value.Equals(_defensiveStance)) return; _defensiveStance = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Fury

        [Setting, DefaultValue(false)]
        public bool FEnableBattleCry
        {
            get { return _fBattleCry; }
            set { if (value.Equals(_fBattleCry)) return; _fBattleCry = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool FBattleCryCd
        {
            get { return _fBattleCryCd; }
            set { if (value.Equals(_fBattleCryCd)) return; _fBattleCryCd = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableOdynsFury
        {
            get { return _odynsFury; }
            set { if (value.Equals(_odynsFury)) return; _odynsFury = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool OdynsFuryCd
        {
            get { return _odynsFuryCd; }
            set { if (value.Equals(_odynsFuryCd)) return; _odynsFuryCd = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool FEnableBladestorm
        {
            get { return _fBladestorm; }
            set { if (value.Equals(_fBladestorm)) return; _fBladestorm = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool FBladestormCd
        {
            get { return _fBladestormCd; }
            set { if (value.Equals(_fBladestormCd)) return; _fBladestormCd = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableDragonRoar
        {
            get { return _dragonRoar; }
            set { if (value.Equals(_dragonRoar)) return; _dragonRoar = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool DragonRoarCd
        {
            get { return _dragonRoar; }
            set { if (value.Equals(_dragonRoarCd)) return; _dragonRoarCd = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableBloodbath
        {
            get { return _bloodBath; }
            set { if (value.Equals(_bloodBath)) return; _bloodBath = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool BloodbathCd
        {
            get { return _fBloodbathCd; }
            set { if (value.Equals(_fBloodbathCd)) return; _fBloodbathCd = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableEnragedRegeneration
        {
            get { return _enragedRegeneration; }
            set { if (value.Equals(_enragedRegeneration)) return; _enragedRegeneration = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool FEnableCommandingShout
        {
            get { return _fCommandingShout; }
            set { if (value.Equals(_fCommandingShout)) return; _fCommandingShout = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool FEnableIntimidatingShout
        {
            get { return _fIntimidatingShout; }
            set { if (value.Equals(_fIntimidatingShout)) return; _fIntimidatingShout = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool FEnableHeroicLeap
        {
            get { return _fHeroicLeap; }
            set { if (value.Equals(_fHeroicLeap)) return; _fHeroicLeap = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool FEnableCharge
        {
            get { return _fCharge; }
            set { if (value.Equals(_fCharge)) return; _fCharge = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool FEnableHeroicThrow
        {
            get { return _fHeroicThrow; }
            set { if (value.Equals(_fHeroicThrow)) return; _fHeroicThrow = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableInnerRage
        {
            get { return _innerRage; }
            set { if (value.Equals(_innerRage)) return; _innerRage = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableFrenzy
        {
            get { return _frenzy; }
            set { if (value.Equals(_frenzy)) return; _frenzy = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Protection

        [Setting, DefaultValue(true)]
        public bool EnableDemoralizingShout
        {
            get { return _demoralizingShout; }
            set { if (value.Equals(_demoralizingShout)) return; _demoralizingShout = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableIgnorePain
        {
            get { return _ignorePain; }
            set { if (value.Equals(_ignorePain)) return; _ignorePain = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableShieldWall
        {
            get { return _shieldWall; }
            set { if (value.Equals(_shieldWall)) return; _shieldWall = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableShieldBlock
        {
            get { return _shieldBlock; }
            set { if (value.Equals(_shieldBlock)) return; _shieldBlock = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableVictoryRush
        {
            get { return _victoryRush; }
            set { if (value.Equals(_victoryRush)) return; _victoryRush = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableNeltharionsRage
        {
            get { return _neltharionsRage; }
            set { if (value.Equals(_neltharionsRage)) return; _neltharionsRage = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool PEnableBattleCry
        {
            get { return _pBattleCry; }
            set { if (value.Equals(_pBattleCry)) return; _pBattleCry = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool PEnableHeroicThrow
        {
            get { return _pHeroicThrow; }
            set { if (value.Equals(_pHeroicThrow)) return; _pHeroicThrow = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool PEnableHeroicLeap
        {
            get { return _pHeroicLeap; }
            set { if (value.Equals(_pHeroicLeap)) return; _pHeroicLeap = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableIntercept
        {
            get { return _intercept; }
            set { if (value.Equals(_intercept)) return; _intercept = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool EnableLastResort
        {
            get { return _lastResort; }
            set { if (value.Equals(_lastResort)) return; _lastResort = value; Notify(); }
        }

        [Setting, DefaultValue(false)]
        public bool EnableShieldReflection
        {
            get { return _shieldReflection; }
            set { if (value.Equals(_shieldReflection)) return; _shieldReflection = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Talents

        [Setting, DefaultValue(true)]
        public bool EnableImpendingVictory
        {
            get { return _impendingVictory; }
            set { if (value.Equals(_impendingVictory)) return; _impendingVictory = value; Notify(); }
        }

        [Setting, DefaultValue(true)]
        public bool PEnableRavager
        {
            get { return _pRavager; }
            set { if (value.Equals(_pRavager)) return; _pRavager = value; Notify(); }
        }

        #endregion

        #endregion

        #region [Settings] - TextBox's

        #region [TabItem] - General
        [Setting, DefaultValue(3)]
        public int AoEValue { get { return _aoeValue; } set { if (value.Equals(_aoeValue)) return; _aoeValue = value; Notify(); } }
        [DefaultValue(false)]
        public bool EnableAoeValue { get { return _autoAoe; } set { _autoAoe = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int ShockwaveValue { get { return _shockwaveValue; } set { if (value.Equals(_shockwaveValue)) return; _shockwaveValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int StormBoltValue { get { return _stormBoltValue; } set { if (value.Equals(_stormBoltValue)) return; _stormBoltValue = value; Notify(); } }
        [Setting, DefaultValue(60)]
        public int RestingValue { get { return _restingValue; } set { if (value.Equals(_restingValue)) return; _restingValue = value; Notify(); } }
        #endregion

        #region [TabItem] - Arms
        [Setting, DefaultValue(0)]
        public int ArmsBurstValue { get { return _aBurstValue; } set { if (value.Equals(_aBurstValue)) return; _aBurstValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int WarbreakerValue { get { return _warbreakerValue; } set { if (value.Equals(_warbreakerValue)) return; _warbreakerValue = value; Notify(); } }
        [Setting, DefaultValue(3)]
        public int ABladestormValue { get { return _aBladestormValue; } set { if (value.Equals(_aBladestormValue)) return; _aBladestormValue = value; Notify(); } }
        [Setting, DefaultValue(3)]
        public int RavagerValue { get { return _ravagerValue; } set { if (value.Equals(_ravagerValue)) return; _ravagerValue = value; Notify(); } }
        [Setting, DefaultValue(30)]
        public int DieByTheSwordValue { get { return _dieByTheSwordValue; } set { if (value.Equals(_dieByTheSwordValue)) return; _dieByTheSwordValue = value; Notify(); } }
        [Setting, DefaultValue(70)]
        public int ACommandingShoutValue { get { return _aCommandingShoutValue; } set { if (value.Equals(_aCommandingShoutValue)) return; _aCommandingShoutValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int AIntimidatingShoutValue { get { return _aIntimidatingShoutValue; } set { if (value.Equals(_aIntimidatingShoutValue)) return; _aIntimidatingShoutValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int DefensiveStanceValue { get { return _defensiveStanceValue; } set { if (value.Equals(_defensiveStanceValue)) return; _defensiveStanceValue = value; Notify(); } }
        #endregion

        #region [TabItem] - Fury
        [Setting, DefaultValue(0)]
        public int FuryBurstValue { get { return _fBurstValue; } set { if (value.Equals(_fBurstValue)) return; _fBurstValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int EnragedRegenerationValue { get { return _enragedRegenerationValue; } set { if (value.Equals(_enragedRegenerationValue)) return; _enragedRegenerationValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int FCommandingShoutValue { get { return _fCommandingShoutValue; } set { if (value.Equals(_fCommandingShoutValue)) return; _fCommandingShoutValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int FIntimidatingShoutValue { get { return _fIntimidatingShoutValue; } set { if (value.Equals(_fIntimidatingShoutValue)) return; _fIntimidatingShoutValue = value; Notify(); } }
        #endregion

        #region [TabItem] - Protection
        [Setting, DefaultValue(0)]
        public int DemoralizingShoutValue { get { return _demoralizingShoutValue; } set { if (value.Equals(_demoralizingShoutValue)) return; _demoralizingShoutValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int ShieldWallValue { get { return _shieldWallValue; } set { if (value.Equals(_shieldWallValue)) return; _shieldWallValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int ShieldBlockValue { get { return _shieldBlockValue; } set { if (value.Equals(_shieldBlockValue)) return; _shieldBlockValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int NeltharionsRageValue { get { return _neltharionsRageValue; } set { if (value.Equals(_neltharionsRageValue)) return; _neltharionsRageValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int PBattleCryValue { get { return _pBattleCryValue; } set { if (value.Equals(_pBattleCryValue)) return; _pBattleCryValue = value; Notify(); } }
        [Setting, DefaultValue(20)]
        public int LastResortValue { get { return _lastResortValue; } set { if (value.Equals(_lastResortValue)) return; _lastResortValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int ShieldReflectionValue { get { return _shieldReflectionValue; } set { if (value.Equals(_shieldReflectionValue)) return; _shieldReflectionValue = value; Notify(); } }
        #endregion

        #region [TabItem] - Talents
        [Setting, DefaultValue(0)]
        public int ImpendingVictoryValue { get { return _impendingVictoryValue; } set { if (value.Equals(_impendingVictoryValue)) return; _impendingVictoryValue = value; Notify(); } }
        [Setting, DefaultValue(0)]
        public int PRavagerValue { get { return _pRavagerValue; } set { if (value.Equals(_pRavagerValue)) return; _pRavagerValue = value; Notify(); } }
        #endregion

        #endregion

        #region [Settings] - ComboBox's

        #region [TabItem] - General

        [Setting, DefaultValue(0)]
        public int ShockwaveCondition
        {
            get { return _shockwaveCondition; }
            set { if (value.Equals(_shockwaveCondition)) return; _shockwaveCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int StormBoltCondition
        {
            get { return _stormBoltCondition; }
            set { if (value.Equals(_stormBoltCondition)) return; _stormBoltCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int AvatarCondition
        {
            get { return _avatarCondition; }
            set { if (value.Equals(_avatarCondition)) return; _avatarCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int BerserkerRageCondition
        {
            get { return _berserkerRageCondition; }
            set { if (value.Equals(_berserkerRageCondition)) return; _berserkerRageCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int PummelCondition
        {
            get { return _pummelCondition; }
            set { if (value.Equals(_pummelCondition)) return; _pummelCondition = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Arms
        [Setting, DefaultValue(0)]
        public int ArmsBurstCondition
        {
            get { return _aBurstCondition; }
            set { if (value.Equals(_aBurstCondition)) return; _aBurstCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int ABladestormCondition
        {
            get { return _aBladestormCondition; }
            set { if (value.Equals(_aBladestormCondition)) return; _aBladestormCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int RavagerCondition
        {
            get { return _ravagerCondition; }
            set { if (value.Equals(_ravagerCondition)) return; _ravagerCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int AIntimidatingShoutCondition
        {
            get { return _aIntimidatingShoutCondition; }
            set { if (value.Equals(_aIntimidatingShoutCondition)) return; _aIntimidatingShoutCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int DefensiveStanceCondition
        {
            get { return _defensiveStanceCondition; }
            set { if (value.Equals(_defensiveStanceCondition)) return; _defensiveStanceCondition = value; Notify(); }
        }
        #endregion

        #region [TabItem] - Fury

        [Setting, DefaultValue(0)]
        public int FuryBurstCondition
        {
            get { return _fBurstCondition; }
            set { if (value.Equals(_fBurstCondition)) return; _fBurstCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int FIntimidatingShoutCondition
        {
            get { return _fIntimidatingShoutCondition; }
            set { if (value.Equals(_fIntimidatingShoutCondition)) return; _fIntimidatingShoutCondition = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Protection

        [Setting, DefaultValue(0)]
        public int PBattleCryCondition
        {
            get { return _pBattleCryCondition; }
            set { if (value.Equals(_pBattleCryCondition)) return; _pBattleCryCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int NeltharionsFuryCondition
        {
            get { return _neltharionsRageCondition; }
            set { if (value.Equals(_neltharionsRageCondition)) return; _neltharionsRageCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int ShieldReflectionCondtion
        {
            get { return _shieldReflectionCondition; }
            set { if (value.Equals(_shieldReflectionCondition)) return; _shieldReflectionCondition = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Talents

        [Setting, DefaultValue(0)]
        public int ImpendingVictoryCondition
        {
            get { return _impendingVictoryCondition; }
            set { if (value.Equals(_impendingVictoryCondition)) return; _impendingVictoryCondition = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int PRavagerCondition
        {
            get { return _pRavagerCondition; }
            set { if (value.Equals(_pRavagerCondition)) return; _pRavagerCondition = value; Notify(); }
        }

        #endregion

        #region [TabItem] - Hotkeys

        [Setting, DefaultValue(0)]
        public int HotkeyAutomationModifier
        {
            get { return _hkAutoModifier; }
            set { _hkAutoModifier = value; Notify(); }
        }

        [DefaultValue(0)]
        public int AutomationKey
        {
            get { return _autoModifierKey; }
            set { _autoModifierKey = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyAbilitiesModifier
        {
            get { return _hotkeyAbilitiesModifier; }
            set { if (value.Equals(_hotkeyAbilitiesModifier)) return; _hotkeyAbilitiesModifier = value; Notify(); }
        }

        [DefaultValue(0)]
        public int AbilitiesKey
        {
            get { return _abilitiesModifierKey; }
            set { if (value.Equals(_abilitiesModifierKey)) return; _abilitiesModifierKey = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyFacing
        {
            get { return _hotkeyFacing; }
            set { if (value.Equals(_hotkeyFacing)) return; _hotkeyFacing = value; Notify(); }
        }

        [DefaultValue(0)]
        public int FacingKey
        {
            get { return _facingKey; }
            set { if (value.Equals(_facingKey)) return; _facingKey = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyTarget
        {
            get { return _hotkeyTarget; }
            set { if (value.Equals(_hotkeyTarget)) return; _hotkeyTarget = value; Notify(); }
        }

        [DefaultValue(0)]
        public int TargetingKey
        {
            get { return _targetKey; }
            set
            { if (value.Equals(_targetKey)) return; _targetKey = value; Notify(); }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyMovement
        {
            get { return _hotkeyMovement; }
            set { if (value.Equals(_hotkeyMovement)) return; _hotkeyMovement = value; Notify(); }
        }

        [DefaultValue(0)]
        public int MovementKey
        {
            get { return _movementKey; }
            set
            {
                if (value.Equals(_movementKey))
                    return;
                _movementKey = value;
                Notify();
            }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyPull
        {
            get { return _hotkeyPull; }
            set
            {
                if (value.Equals(_hotkeyPull))
                    return;
                _hotkeyPull = value;
                Notify();
            }
        }

        [DefaultValue(0)]
        public int PullKey
        {
            get { return _pullKey; }
            set
            {
                if (value.Equals(_pullKey))
                    return;
                _pullKey = value;
                Notify();
            }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyAoE
        {
            get { return _hotkeyAoE; }
            set
            {
                if (value.Equals(_hotkeyAoE))
                    return;
                _hotkeyAoE = value;
                Notify();
            }
        }

        [DefaultValue(0)]
        public int AoEKey
        {
            get { return _aoeKey; }
            set
            {
                if (value.Equals(_aoeKey))
                    return;
                _aoeKey = value;
                Notify();
            }
        }

        [Setting, DefaultValue(0)]
        public int HotkeyBurst
        {
            get { return _hotkeyBurst; }
            set
            {
                if (value.Equals(_hotkeyBurst))
                    return;
                _hotkeyBurst = value;
                Notify();
            }
        }

        [DefaultValue(0)]
        public int BurstKey
        {
            get { return _burstKey; }
            set
            {
                if (value.Equals(_burstKey))
                    return;
                _burstKey = value;
                Notify();
            }
        }

        #endregion

        #endregion

        #endregion

        public void ConditionInit()
        {
            Shockwave = (AoEStunType) ConditionInitialize(ShockwaveCondition, "AoEStunType", Shockwave);
            StormBolt = (SingleStunType)ConditionInitialize(StormBoltCondition, "SingleStunType", StormBolt);
            Pummel = (InterruptType)ConditionInitialize(PummelCondition, "InterruptType", Pummel);
            Avatar = (ClassType)ConditionInitialize(AvatarCondition, "CooldownType", Avatar);
            BerserkerRage = (ClassType)ConditionInitialize(BerserkerRageCondition, "CooldownType", BerserkerRage);
            ArmsBurst = (BurstType)ConditionInitialize(ArmsBurstCondition, "BurstType", ArmsBurst);
            ABladestorm = (ArmsType)ConditionInitialize(ABladestormCondition, "ArmsType", ABladestorm);
            Ravager = (ArmsType)ConditionInitialize(RavagerCondition, "ArmsType", Ravager);
            FuryBurst = (BurstType)ConditionInitialize(FuryBurstCondition, "BurstType", FuryBurst);
            AIntimidatingShout = (CrowdControlType)ConditionInitialize(AIntimidatingShoutCondition, "CrowdControlType", AIntimidatingShout);
            DefensiveStance = (DefenseType)ConditionInitialize(DefensiveStanceCondition, "DefenseTypeTwo", DefensiveStance);
            PRavager = (DefenseType)ConditionInitialize(PRavagerCondition, "DefenseType", PRavager);                       
            FIntimidatingShout = (CrowdControlType)ConditionInitialize(FIntimidatingShoutCondition, "CrowdControlType", FIntimidatingShout);
            PBattleCry = (BuffType)ConditionInitialize(PBattleCryCondition, "BuffType", PBattleCry);
            ImpendingVictory = (ImpendingVictoryType)ConditionInitialize(ImpendingVictoryCondition, "ImpendingVictoryType", ImpendingVictory);
            NeltharionsRage = (BuffType)ConditionInitialize(NeltharionsFuryCondition, "BuffType", NeltharionsRage);
            ShieldReflection = (BuffType)ConditionInitialize(ShieldReflectionCondtion, "BuffType", ShieldReflection);

            #region Commented Out

            //DieByTheSword = (DefenseType)ConditionInitialize(DieByTheSwordCondition, "DefenseType", DieByTheSword);
            //ACommandingShout = (DefenseType)ConditionInitialize(ACommandingShoutCondition, "DefenseType", ACommandingShout);
            //EnragedRegeneration = (DefenseType)ConditionInitialize(EnragedRegenerationCondition, "DefenseType", EnragedRegeneration);
            //FCommandingShout = (DefenseType)ConditionInitialize(FCommandingShoutCondition, "DefenseType", FCommandingShout);

            #endregion
        }

        public void HotkeyInit()
        {
            if (HotkeyAutomationModifier == 0)
                AutomationKey = 0;
            if (HotkeyAutomationModifier == 1)
                AutomationKey = 1;
            if (HotkeyAutomationModifier == 2)
                AutomationKey = 2;
            if (HotkeyAutomationModifier == 3)
                AutomationKey = 4;
            if (HotkeyAbilitiesModifier == 0)
                AbilitiesKey = 0;
            if (HotkeyAbilitiesModifier == 1)
                AbilitiesKey = 1;
            if (HotkeyAbilitiesModifier == 2)
                AbilitiesKey = 2;
            if (HotkeyAbilitiesModifier == 3)
                AbilitiesKey = 4;

            FacingKey = HotkeyInit(HotkeyFacing, FacingKey);
            TargetingKey = HotkeyInit(HotkeyTarget, TargetingKey);
            MovementKey = HotkeyInit(HotkeyMovement, MovementKey);
            PullKey = HotkeyInit(HotkeyPull, PullKey);
            AoEKey = HotkeyInit(HotkeyAoE, AoEKey);
            BurstKey = HotkeyInit(HotkeyBurst, BurstKey);
        }

        public int HotkeyInit(int hotkey, int key)
        {
            if (hotkey != 0 && hotkey <= 26)
            {
                if (hotkey == 1)
                    key = 65;
                if (hotkey == 2)
                    key = 66;
                if (hotkey == 3)
                    key = 67;
                if (hotkey == 4)
                    key = 68;
                if (hotkey == 5)
                    key = 69;
                if (hotkey == 6)
                    key = 70;
                if (hotkey == 7)
                    key = 71;
                if (hotkey == 8)
                    key = 72;
                if (hotkey == 9)
                    key = 73;
                if (hotkey == 10)
                    key = 74;
                if (hotkey == 11)
                    key = 75;
                if (hotkey == 12)
                    key = 76;
                if (hotkey == 13)
                    key = 77;
                if (hotkey == 14)
                    key = 78;
                if (hotkey == 15)
                    key = 79;
                if (hotkey == 16)
                    key = 80;
                if (hotkey == 17)
                    key = 81;
                if (hotkey == 18)
                    key = 82;
                if (hotkey == 19)
                    key = 83;
                if (hotkey == 20)
                    key = 84;
                if (hotkey == 21)
                    key = 85;
                if (hotkey == 22)
                    key = 86;
                if (hotkey == 23)
                    key = 87;
                if (hotkey == 24)
                    key = 88;
                if (hotkey == 25)
                    key = 89;
                if (hotkey == 26)
                    key = 90;
                if (hotkey == 27)
                    key = 112;
                if (hotkey == 28)
                    key = 113;
                if (hotkey == 29)
                    key = 114;
                if (hotkey == 30)
                    key = 115;
                if (hotkey == 31)
                    key = 116;
                if (hotkey == 32)
                    key = 117;
                if (hotkey == 33)
                    key = 118;
                if (hotkey == 34)
                    key = 119;
                if (hotkey == 35)
                    key = 120;
                if (hotkey == 36)
                    key = 121;
                if (hotkey == 37)
                    key = 122;
                if (hotkey == 38)
                    key = 123;

                return key;
            }
            return 0;
        }

        public Enum ConditionInitialize(int index, string type, Enum e)
        {
            switch (type)
            {
                case "AoEStunType":
                    if (index == 0) { Shockwave = AoEStunType.OnCooldown;}
                    if (index == 1) { Shockwave = AoEStunType.OnBurst; }
                    if (index == 2) { Shockwave = AoEStunType.HpBelow; }
                    if (index == 3) { Shockwave = AoEStunType.TargetCount; }
                    if (index == 4) { Shockwave = AoEStunType.TargetCastAny; }
                    if (index == 5) { Shockwave = AoEStunType.TargetCastHeal; }
                    return Shockwave;
                case "SingleStunType":
                    if (index == 0) { StormBolt = SingleStunType.OnCooldown; }
                    if (index == 1) { StormBolt = SingleStunType.HpBelow; }
                    if (index == 2) { StormBolt = SingleStunType.TargetCastAny; }
                    if (index == 3) { StormBolt = SingleStunType.TargetCastHeal; }
                    return StormBolt;
                case "InterruptType":
                    if (index == 0) { Pummel = InterruptType.TargetCastAny; }
                    if (index == 1) { Pummel = InterruptType.TargetCastHeal; }
                    return Pummel;
                case "CooldownType":
                    if (index == 0) { e = ClassType.OnCooldown; }
                    if (index == 1) { e = ClassType.OnBurst; }
                    if (index == 2) { e = ClassType.PlayerBossElite; }
                    return e;
                case "DefenseType":
                    if (index == 0) { e = DefenseType.HpBelow; }
                    if (index == 1) { e = DefenseType.OnCooldown; }
                    if (index == 2) { e = DefenseType.TargetCount; }
                    return e;
                case "DefenseTypeTwo":
                    if (index == 0) { e = DefenseType.HpBelow; }
                    if (index == 1) { e = DefenseType.TargetCount; }
                    return e;
                case "BurstType":
                    if (index == 0) { e = BurstType.Never; }
                    if (index == 1) { e = BurstType.PlayerBossElite; }
                    if (index == 2) { e = BurstType.BurstKeys; }
                    if (index == 3) { e = BurstType.HpBelow; }
                    if (index == 4) { e = BurstType.OnCooldown; }
                    if (index == 5) { e = BurstType.TargetCount; }
                    if (index == 6) { e = BurstType.TargetHpBelow; }
                    return e;
                case "ArmsType":
                    if (index == 0) { e = ArmsType.OnBurst; }
                    if (index == 1) { e = ArmsType.OnCooldown; }
                    if (index == 2) { e = ArmsType.TargetCount; }
                    return e;
                case "CrowdControlType":
                    if (index == 0) { e = CrowdControlType.HpBelow; }
                    if (index == 1) { e = CrowdControlType.TargetCount; }
                    if (index == 2) { e = CrowdControlType.TargetCastAny; }
                    if (index == 3) { e = CrowdControlType.TargetCastHeal; }
                    return e;
                case "BuffType":
                    if (index == 0) { e = BuffType.HpBelow; }
                    if (index == 1) { e = BuffType.PlayerBossElite; }
                    if (index == 2) { e = BuffType.OnCooldown; }
                    if (index == 3) { e = BuffType.TargetCount; }
                    return e;
                case "ImpendingVictoryType":
                    if (index == 0) { ImpendingVictory = ImpendingVictoryType.HpBelow; }
                    if (index == 1) { ImpendingVictory = ImpendingVictoryType.PlayerBossElite; }
                    if (index == 2) { ImpendingVictory = ImpendingVictoryType.OnCooldown; }
                    return ImpendingVictory;
            }
            return null;
        }

        private void Notify()
        {
            StackFrame[] frames = new StackTrace(false).GetFrames();
            if (frames != null)
            {
                string name = frames[1].GetMethod().Name;
                if (name.StartsWith("set_"))
                    name = name.Replace("set_", "");
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
            }
        }

        #region [Properties] - Class Functions

        #region Interrupts

        public bool EnableInterrupt => EnablePummel
            || (EnableShockwave && (Shockwave == AoEStunType.TargetCastAny || Shockwave == AoEStunType.TargetCastHeal))
            || (EnableStormBolt && (StormBolt == SingleStunType.TargetCastAny || StormBolt == SingleStunType.TargetCastHeal))
            || (AEnableIntimidatingShout && (AIntimidatingShout == CrowdControlType.TargetCastAny || AIntimidatingShout == CrowdControlType.TargetCastHeal))
            || (FEnableIntimidatingShout && (FIntimidatingShout == CrowdControlType.TargetCastAny || FIntimidatingShout == CrowdControlType.TargetCastHeal));

        public bool AllowArmInterrupts => ArmsInterruptAny || ArmsInterruptHeal;
        public bool AllowFuryInterrupts => FuryInterruptAny || FuryInterruptHeal;
        public bool AllowProtInterrupts => ProtInterruptAny || ProtInterruptHeal;

        public bool ArmsInterruptHeal => PummelHeal || ShockwaveHeal || StormBoltHeal || ArmsIntimidatingShoutHeal;
        public bool FuryInterruptHeal => PummelHeal || ShockwaveHeal || StormBoltHeal || FuryIntimidatingShoutHeal;
        public bool ProtInterruptHeal => PummelHeal || ShockwaveHeal || StormBoltHeal;
        public bool ArmsInterruptAny => PummelAny || ShockwaveAny || StormBoltAny || ArmsIntimidatingShoutAny;
        public bool FuryInterruptAny => PummelAny || ShockwaveAny || StormBoltAny || FuryIntimidatingShoutAny;
        public bool ProtInterruptAny => PummelAny || ShockwaveAny || StormBoltAny;

        public bool PummelHeal => EnablePummel && Pummel == InterruptType.TargetCastHeal;
        public bool PummelAny => EnablePummel && Pummel == InterruptType.TargetCastAny;
        public bool ShockwaveHeal => EnableShockwave && Shockwave == AoEStunType.TargetCastHeal;
        public bool ShockwaveAny => EnableShockwave && Shockwave == AoEStunType.TargetCastAny;
        public bool StormBoltHeal => EnableStormBolt && StormBolt == SingleStunType.TargetCastHeal;
        public bool StormBoltAny => EnableStormBolt && StormBolt == SingleStunType.TargetCastAny;
        public bool ArmsIntimidatingShoutHeal => AEnableIntimidatingShout && AIntimidatingShout == CrowdControlType.TargetCastHeal;
        public bool ArmsIntimidatingShoutAny => AEnableIntimidatingShout && AIntimidatingShout == CrowdControlType.TargetCastAny;
        public bool FuryIntimidatingShoutHeal => FEnableIntimidatingShout && FIntimidatingShout == CrowdControlType.TargetCastHeal;
        public bool FuryIntimidatingShoutAny => FEnableIntimidatingShout && FIntimidatingShout == CrowdControlType.TargetCastAny;

        #endregion

        #region Defensive Cooldowns

        public bool EnableArmsDefensive => EnableDieByTheSword || AEnableCommandingShout 
            || (AEnableIntimidatingShout && (AIntimidatingShout == CrowdControlType.HpBelow || AIntimidatingShout == CrowdControlType.TargetCount))
            || EnableDefensiveStance
            || (EnableShockwave && (Shockwave == AoEStunType.HpBelow || Shockwave == AoEStunType.TargetCount))
            || EnableStormBolt && StormBolt == SingleStunType.HpBelow;

        public bool EnableFuryDefensive => EnableEnragedRegeneration || FEnableCommandingShout
            || (FEnableIntimidatingShout && (FIntimidatingShout == CrowdControlType.HpBelow || FIntimidatingShout == CrowdControlType.TargetCount))
            || (EnableShockwave && (Shockwave == AoEStunType.HpBelow || Shockwave == AoEStunType.TargetCount))
            || EnableStormBolt && StormBolt == SingleStunType.HpBelow;

        public bool EnableProtDefensive => EnableDemoralizingShout || EnableIgnorePain || EnableShieldBlock || EnableShieldWall || EnableVictoryRush
            || PEnableBattleCry || EnableBerserkerRage
            || (EnableShockwave && (Shockwave == AoEStunType.HpBelow || Shockwave == AoEStunType.TargetCount))
            || EnableStormBolt && StormBolt == SingleStunType.HpBelow;

        #endregion

        #region Offensive Cooldowns

        public bool EnableArmsOffensive => (EnableShockwave && (Shockwave == AoEStunType.OnBurst || Shockwave == AoEStunType.OnCooldown))
            || EnableStormBolt && StormBolt == SingleStunType.OnCooldown
            || (EnableAvatar && (Avatar == ClassType.OnBurst || Avatar == ClassType.OnCooldown || Avatar == ClassType.PlayerBossElite))
            || AEnableBattleCry || AEnableBladeStorm || ArmsBurst != BurstType.Never || EnableWarbreaker;

        public bool EnableFuryOffensive => (EnableShockwave && (Shockwave == AoEStunType.OnBurst || Shockwave == AoEStunType.OnCooldown))
            || EnableStormBolt && StormBolt == SingleStunType.OnCooldown
            || (EnableAvatar && (Avatar == ClassType.OnBurst || Avatar == ClassType.OnCooldown || Avatar == ClassType.PlayerBossElite))
            || FuryBurst != BurstType.Never || FEnableBattleCry || EnableOdynsFury
            || FEnableBladestorm
            || EnableDragonRoar
            || EnableBloodbath;

        public bool EnableProtOffensive => (EnableShockwave && (Shockwave == AoEStunType.OnBurst || Shockwave == AoEStunType.OnCooldown))
            || EnableStormBolt && StormBolt == SingleStunType.OnCooldown
            || (EnableAvatar && (Avatar == ClassType.OnBurst || Avatar == ClassType.OnCooldown || Avatar == ClassType.PlayerBossElite))
            || EnableImpendingVictory || PEnableRavager;

        #endregion

        #endregion
    }
}
